import argparse

from apigee.api.apps import Apps

from apigee.parsers.parent_parser import ParentParser
from apigee.parsers.file_parser import FileParser
from apigee.parsers.prefix_parser import PrefixParser
from apigee.parsers.silent_parser import SilentParser
from apigee.parsers.verbose_parser import VerboseParser

from apigee.util import console


class ParserApps:
    def __init__(self, parser, **kwargs):
        self._parser = parser
        self._parser_apps = self._parser.add_parser(
            "apps", help="manage developer apps"
        ).add_subparsers()
        self._parent_parser = kwargs.get("parent_parser", ParentParser())
        self._file_parser = kwargs.get("file_parser", FileParser())
        self._prefix_parser = kwargs.get(
            "prefix_parser", PrefixParser(profile="default")
        )
        self._silent_parser = kwargs.get("silent_parser", SilentParser())
        self._verbose_parser = kwargs.get("verbose_parser", VerboseParser())
        self._create_parser()

    @property
    def parser(self):
        return self._parser

    @parser.setter
    def parser(self, value):
        self._parser = value

    @property
    def parser_apps(self):
        return self._parser_apps

    @parser_apps.setter
    def parser_apps(self, value):
        self._parser_apps = value

    @property
    def parent_parser(self):
        return self._parent_parser

    @parent_parser.setter
    def parent_parser(self, value):
        self._parent_parser = value

    @property
    def file_parser(self):
        return self._file_parser

    @file_parser.setter
    def file_parser(self, value):
        self._file_parser = value

    @property
    def prefix_parser(self):
        return self._prefix_parser

    @prefix_parser.setter
    def prefix_parser(self, value):
        self._prefix_parser = value

    def __call__(self):
        return self._parser

    def _build_create_developer_app_argument(self):
        create_developer_app = self._parser_apps.add_parser(
            "create",
            aliases=["create-developer-app"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Creates an app associated with a developer, associates the app with an API product, and auto-generates an API key for the app to use in calls to API proxies inside the API product.",
        )
        create_developer_app.add_argument(
            "-d", "--developer", help="developer email or id", required=True
        )
        create_developer_app.add_argument(
            "-b", "--body", help="request body", required=True
        )
        create_developer_app.set_defaults(
            func=lambda args: console.log(
                Apps(args, args.org, None)
                .create_developer_app(args.developer, args.body)
                .text
            )
        )

    def _build_delete_developer_app_arguments(self):
        delete_developer_app = self._parser_apps.add_parser(
            "delete",
            aliases=["delete-developer-app"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Deletes a developer app.",
        )
        delete_developer_app.add_argument(
            "-d", "--developer", help="developer email or id", required=True
        )
        delete_developer_app.add_argument(
            "-n",
            "--name",
            help="Name of the app. The name is the unique ID of the app for this organization and developer.",
            required=True,
        )
        delete_developer_app.set_defaults(
            func=lambda args: console.log(
                Apps(args, args.org, args.name)
                .delete_developer_app(args.developer)
                .text
            )
        )

    def _build_create_empty_developer_app_argument(self):
        create_empty_developer_app = self._parser_apps.add_parser(
            "create-empty",
            aliases=["create-empty-developer-app"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Creates an empty developer app.",
        )
        create_empty_developer_app.add_argument(
            "-d", "--developer", help="developer email or id", required=True
        )
        create_empty_developer_app.add_argument(
            "-n",
            "--name",
            help="Name of the app. The name is the unique ID of the app for this organization and developer.",
            required=True,
        )
        # create_empty_developer_app.add_argument('--products', help='A list of API products associated with the app\'s credentials', nargs='+', required=False, default=[])
        create_empty_developer_app.add_argument(
            "--display-name",
            help="The DisplayName (set with an attribute) is what appears in the management UI. If you don't provide a DisplayName, the name is used.",
            required=False,
            default=None,
        )
        # create_empty_developer_app.add_argument('--scopes', help='Sets the scopes element under the apiProducts element in the attributes of the app. The specified scopes must already exist on the API products associated with the app.', nargs='+', required=False, default=[])
        create_empty_developer_app.add_argument(
            "--callback-url",
            help="The callbackUrl is used by OAuth 2.0 authorization servers to communicate authorization codes back to apps. CallbackUrl must match the value of redirect_uri in some OAuth 2.0 See the documentation on OAuth 2.0 for more details.",
            required=False,
            default=None,
        )
        create_empty_developer_app.set_defaults(
            func=lambda args: console.log(
                Apps(args, args.org, args.name)
                .create_empty_developer_app(
                    args.developer,
                    display_name=args.display_name,
                    callback_url=args.callback_url,
                )
                .text
            )
        )

    def _build_create_a_consumer_key_and_secret_argument(self):
        create_a_consumer_key_and_secret = self._parser_apps.add_parser(
            "create-creds",
            aliases=["create-a-consumer-key-and-secret", "create-credentials"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Creates a custom consumer key and secret for a developer app. This is particularly useful if you want to migrate existing consumer keys/secrets to Edge from another system. Consumer keys and secrets can contain letters, numbers, underscores, and hyphens. No other special characters are allowed.",
        )
        create_a_consumer_key_and_secret.add_argument(
            "-d", "--developer", help="developer email or id", required=True
        )
        create_a_consumer_key_and_secret.add_argument(
            "-n",
            "--name",
            help="Name of the app. The name is the unique ID of the app for this organization and developer.",
            required=True,
        )
        create_a_consumer_key_and_secret.add_argument(
            "--consumer-key", help="", required=False, default=None
        )
        create_a_consumer_key_and_secret.add_argument(
            "--consumer-secret", help="", required=False, default=None
        )
        create_a_consumer_key_and_secret.add_argument(
            "--key-length",
            help="length of consumer key",
            required=False,
            type=int,
            default=32,
        )
        create_a_consumer_key_and_secret.add_argument(
            "--secret-length",
            help="length of consumer secret",
            required=False,
            type=int,
            default=32,
        )
        create_a_consumer_key_and_secret.add_argument(
            "--key-suffix", help="", required=False, default=None
        )
        create_a_consumer_key_and_secret.add_argument(
            "--key-delimiter",
            help="separates consumerKey and key suffix with a delimiter. the default is '-'.",
            required=False,
            default="-",
        )
        create_a_consumer_key_and_secret.add_argument(
            "--products",
            help="A list of API products to be associated with the app's credentials",
            nargs="+",
            required=False,
            default=[],
        )
        create_a_consumer_key_and_secret.set_defaults(
            func=lambda args: console.log(
                Apps(args, args.org, args.name)
                .create_a_consumer_key_and_secret(
                    args.developer,
                    consumer_key=args.consumer_key,
                    consumer_secret=args.consumer_secret,
                    key_length=args.key_length,
                    secret_length=args.secret_length,
                    key_suffix=args.key_suffix,
                    key_delimiter=args.key_delimiter,
                    products=args.products,
                )
                .text
            )
        )

    def _build_list_developer_apps_argument(self):
        list_developer_apps = self._parser_apps.add_parser(
            "list",
            aliases=["list-developer-apps"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._prefix_parser(),
            ],
            help="Lists all apps created by a developer in an organization, and optionally provides an expanded view of the apps. All time values in the response are UNIX times. You can specify either the developer's email address or Edge ID.",
        )
        list_developer_apps.add_argument(
            "-d", "--developer", help="developer email or id", required=True
        )
        list_developer_apps.add_argument(
            "--expand",
            action="store_true",
            help="Set to true to expand the results. This query parameter does not work if you use the count or startKey query parameters.",
        )
        list_developer_apps.add_argument(
            "--count",
            default=100,
            type=int,
            help="Limits the list to the number you specify. The limit is 100. Use with the startKey parameter to provide more targeted filtering.",
        )
        list_developer_apps.add_argument(
            "--startkey",
            default="",
            help="To filter the keys that are returned, enter the name of a company app that the list will start with.",
        )
        list_developer_apps.set_defaults(
            func=lambda args: console.log(
                Apps(args, args.org, None).list_developer_apps(
                    args.developer,
                    prefix=args.prefix,
                    expand=args.expand,
                    count=args.count,
                    startkey=args.startkey,
                )
            )
        )

    def _build_get_developer_app_details_argument(self):
        get_developer_app_details = self._parser_apps.add_parser(
            "get",
            aliases=["get-developer-app-details"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Get the profile of a specific developer app. All times in the response are UNIX times. Note that the response contains a top-level attribute named accessType that is no longer used by Apigee.",
        )
        get_developer_app_details.add_argument(
            "-d", "--developer", help="developer email or id", required=True
        )
        get_developer_app_details.add_argument(
            "-n", "--name", help="name", required=True
        )
        get_developer_app_details.set_defaults(
            func=lambda args: console.log(
                Apps(args, args.org, args.name)
                .get_developer_app_details(args.developer)
                .text
            )
        )

    def _build_restore_app_argument(self):
        restore_app = self._parser_apps.add_parser(
            "restore",
            aliases=["restore-app"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._file_parser(),
            ],
            help="Restore developer app from a file.",
        )
        restore_app.set_defaults(
            func=lambda args: console.log(
                Apps(args, args.org, None).restore_app(args.file).text
            )
        )

    def _create_parser(self):
        self._build_create_developer_app_argument()
        self._build_delete_developer_app_arguments()
        self._build_create_empty_developer_app_argument()
        self._build_create_a_consumer_key_and_secret_argument()
        self._build_list_developer_apps_argument()
        self._build_get_developer_app_details_argument()
        self._build_restore_app_argument()



import binascii
import pyotp
import requests
import sys
import urllib.request, urllib.parse, urllib.error
from requests.adapters import HTTPAdapter
from requests.exceptions import ConnectionError
from requests.packages.urllib3.util.retry import Retry

from apigee import *
from apigee import APIGEE_OAUTH_URL

from apigee.util import console


def get_access_token(
    args, retries=4, backoff_factor=0.3, status_forcelist=(500, 502, 504), session=None
):
    if args.mfa_secret is None:
        return
    APIGEE_USERNAME = args.username
    APIGEE_PASSWORD = args.password
    APIGEE_MFA_SECRET = args.mfa_secret
    TOTP = pyotp.TOTP(APIGEE_MFA_SECRET)
    try:
        TOTP.now()
    except binascii.Error as e:
        sys.exit(f"{type(e).__name__}: {e}: Not a valid MFA key")
    retry = Retry(
        total=retries,
        read=retries,
        connect=retries,
        backoff_factor=backoff_factor,
        status_forcelist=status_forcelist,
    )
    adapter = HTTPAdapter(max_retries=retry)
    session = requests.Session()
    session.mount("https://", adapter)
    post_headers = {
        "Content-Type": "application/x-www-form-urlencoded;charset=utf-8",
        "Accept": "application/json;charset=utf-8",
        "Authorization": "Basic ZWRnZWNsaTplZGdlY2xpc2VjcmV0",
    }
    post_body = f"username={urllib.parse.quote(APIGEE_USERNAME)}&password={urllib.parse.quote(APIGEE_PASSWORD)}&grant_type=password"
    try:
        response_post = session.post(
            f"{APIGEE_OAUTH_URL}?mfa_token={TOTP.now()}",
            headers=post_headers,
            data=post_body,
        )
    except ConnectionError as ce:
        console.log(ce)
    try:
        response_post.json()["access_token"]
    except KeyError as ke:
        # console.log(f"retry http POST {APIGEE_OAUTH_URL}")
        response_post = session.post(
            f"{APIGEE_OAUTH_URL}?mfa_token={TOTP.now()}",
            headers=post_headers,
            data=post_body,
        )
        # console.log(response_post.json())
    return response_post.json()["access_token"]



import argparse
import builtins
import http.client as http_client
import logging
import requests


class VerboseParser:
    def __init__(self):
        self._parent_parser = argparse.ArgumentParser(add_help=False)
        self._build_verbose_argument()
        self._verbose = self._parent_parser.parse_known_args()[0].verbose
        builtins.APIGEE_CLI_TOGGLE_VERBOSE = self._verbose
        if self._verbose:
            http_client.HTTPConnection.debuglevel = self._verbose - 1
            logging.basicConfig()
            logging.getLogger().setLevel(logging.DEBUG)
            requests_log = logging.getLogger("requests.packages.urllib3")
            requests_log.setLevel(logging.DEBUG)
            requests_log.propagate = True
        self._create_parser()

    @property
    def parent_parser(self):
        return self._parent_parser

    @parent_parser.setter
    def parent_parser(self, value):
        self._parent_parser = value

    def __call__(self):
        return self._parent_parser

    def _build_verbose_argument(self):
        self._parent_parser.add_argument(
            "-v",
            "--verbose",
            action="count",
            help="toggle verbose output",
            required=False,
            default=0,
        )

    def _create_parser(self):
        pass



import argparse

from apigee.util.types import ArgumentType


class FileParser:
    def __init__(self):
        self._parent_parser = argparse.ArgumentParser(add_help=False)
        self._create_parser()

    @property
    def parent_parser(self):
        return self._parent_parser

    @parent_parser.setter
    def parent_parser(self, value):
        self._parent_parser = value

    def __call__(self):
        return self._parent_parser

    def _build_file_argument(self):
        self._parent_parser.add_argument(
            "-f",
            "--file",
            action="store",
            help="file path",
            required=True,
            type=ArgumentType.isfile,
        )

    def _create_parser(self):
        self._build_file_argument()



#!/usr/bin/env python3

import json
import os
from pathlib import Path
from requests.exceptions import HTTPError

from apigee.api.apis import Apis
from apigee.api.keyvaluemaps import Keyvaluemaps
from apigee.api.targetservers import Targetservers
from apigee.api.caches import Caches
from apigee.api.developers import Developers
from apigee.api.apps import Apps
from apigee.api.apiproducts import Apiproducts
from apigee.api.permissions import Permissions
from apigee.api.userroles import Userroles
from apigee.util import authorization, console
from apigee.util.os import touch, extractzip, isfile, isdir, read_file, write_file
from apigee.util.types import Struct


def list_proxies(auth, org, prefix=None):
    return json.loads(Apis(auth, org, None).list_api_proxies(prefix=prefix))


def get_proxy(auth, org, api_name):
    return Apis(auth, org, None).get_api_proxy(api_name)


def export_proxy(auth, org, api_name, revision_number, fs_write=True, output_file=None):
    return Apis(auth, org, None).export_api_proxy(
        api_name, revision_number, fs_write=fs_write, output_file=output_file
    )


def list_kvms(auth, org, environment, prefix=None):
    return json.loads(
        Keyvaluemaps(auth, org, None).list_keyvaluemaps_in_an_environment(
            environment, prefix=prefix
        )
    )


def get_kvm(auth, org, environment, kvm_name):
    return Keyvaluemaps(auth, org, kvm_name).get_keyvaluemap_in_an_environment(
        environment
    )


def list_targetservers(auth, org, environment, prefix=None):
    return json.loads(
        Targetservers(auth, org, None).list_targetservers_in_an_environment(
            environment, prefix=prefix
        )
    )


def get_targetserver(auth, org, environment, targetserver_name):
    return Targetservers(auth, org, targetserver_name).get_targetserver(environment)


def list_caches(auth, org, environment, prefix=None):
    return json.loads(
        Caches(auth, org, None).list_caches_in_an_environment(
            environment, prefix=prefix
        )
    )


def get_cache(auth, org, environment, cache_name):
    return Caches(auth, org, cache_name).get_information_about_a_cache(environment)


def list_developers(auth, org, prefix=None, expand=False, count=1000, startkey=""):
    return json.loads(
        Developers(auth, org, None).list_developers(
            prefix=prefix, expand=expand, count=count, startkey=startkey,
        )
    )


def get_developer(auth, org, developer_name):
    return Developers(auth, org, developer_name).get_developer()


def list_apps(auth, org, prefix=None, expand=False, count=1000, startkey=""):
    apps = {}
    apps_obj = Apps(auth, org, None)
    for dev in list_developers(
        auth, org, prefix=prefix, expand=expand, count=count, startkey=startkey,
    ):
        apps[dev] = json.loads(
            apps_obj.list_developer_apps(
                dev, prefix=None, expand=expand, count=1000, startkey=startkey
            )
        )
    return apps


def get_app(auth, org, developer_name, app_name):
    return Apps(auth, org, app_name).get_developer_app_details(developer_name)


def list_products(auth, org, prefix=None, expand=False, count=1000, startkey=""):
    return json.loads(
        Apiproducts(auth, org, None).list_api_products(
            prefix=None, expand=expand, count=1000, startkey=startkey
        )
    )


def get_product(auth, org, product_name):
    return Apiproducts(auth, org, product_name).get_api_product()


def list_roles(auth, org, prefix=None):
    return Userroles(auth, org, None).list_user_roles().json()


def get_users_for_role(auth, org, role_name):
    try:
        return Userroles(auth, org, role_name).get_users_for_a_role().text
    except HTTPError as e:
        return str(e)


def get_permissions(auth, org, role_name):
    try:
        return (
            Permissions(auth, org, role_name)
            .get_permissions(formatted=True, format="json")
            .text
        )
    except HTTPError as e:
        return str(e)


def snapshot_apis(
    auth,
    org,
    struct,
    environments=["test", "prod"],
    prefix=None,
    fs_write=False,
    base_dir="snapshots",
    apis_dir="apis",
):
    for api in list_proxies(auth, org, prefix=None):
        struct.apis[api] = get_proxy(auth, org, api).json()
        write_file(
            struct.apis[api],
            str(Path(org) / base_dir / apis_dir / (api + ".json")),
            fs_write=fs_write,
        )
    return struct


def snapshot_kvms(
    auth,
    org,
    struct,
    environment,
    prefix=None,
    fs_write=False,
    base_dir="snapshots",
    kvms_dir="keyvaluemaps",
):
    try:
        struct.kvms[environment] = list_kvms(auth, org, environment, prefix=prefix)
    except HTTPError:
        struct.kvms[environment] = []
    write_file(
        struct.kvms[environment],
        str(Path(org) / base_dir / kvms_dir / environment / "keyvaluemaps.json"),
        fs_write=fs_write,
    )
    return struct


def snapshot_targetservers(
    auth,
    org,
    struct,
    environment,
    prefix=None,
    fs_write=False,
    base_dir="snapshots",
    targetservers_dir="targetservers",
):
    try:
        struct.targetservers[environment] = list_targetservers(
            auth, org, environment, prefix=prefix
        )
    except HTTPError:
        struct.targetservers[environment] = []
    write_file(
        struct.targetservers[environment],
        str(
            Path(org)
            / base_dir
            / targetservers_dir
            / environment
            / "targetservers.json"
        ),
        fs_write=fs_write,
    )
    return struct


def snapshot_caches(
    auth,
    org,
    struct,
    environment,
    prefix=None,
    fs_write=False,
    base_dir="snapshots",
    caches_dir="caches",
):
    try:
        struct.caches[environment] = list_caches(auth, org, environment, prefix=prefix)
    except HTTPError:
        struct.caches[environment] = []
    write_file(
        struct.caches[environment],
        str(Path(org) / base_dir / caches_dir / environment / "caches.json"),
        fs_write=fs_write,
    )
    return struct


def snapshot_developers(
    auth,
    org,
    struct,
    prefix=None,
    fs_write=False,
    base_dir="snapshots",
    developers_dir="developers",
):
    struct.developers = list_developers(auth, org, prefix=None)
    write_file(
        struct.developers,
        str(Path(org) / base_dir / developers_dir / "developers.json"),
        fs_write=fs_write,
    )
    return struct


def snapshot_products(
    auth,
    org,
    struct,
    prefix=None,
    fs_write=False,
    base_dir="snapshots",
    products_dir="apiproducts",
):
    struct.products = list_products(auth, org, prefix=None)
    write_file(
        struct.products,
        str(Path(org) / base_dir / products_dir / "apiproducts.json"),
        fs_write=fs_write,
    )
    return struct


def snapshot_apps(
    auth,
    org,
    struct,
    prefix=None,
    fs_write=False,
    base_dir="snapshots",
    apps_dir="apps",
):
    struct.apps = list_apps(auth, org, prefix=None)
    for k, v in struct.apps.items():
        write_file(
            v, str(Path(org) / base_dir / apps_dir / (k + ".json")), fs_write=fs_write,
        )
    return struct


def snapshot_roles(
    auth,
    org,
    struct,
    prefix=None,
    fs_write=False,
    base_dir="snapshots",
    roles_dir="userroles",
):
    struct.roles = list_roles(auth, org, prefix=None)
    write_file(
        struct.roles,
        str(Path(org) / base_dir / roles_dir / "userroles.json"),
        fs_write=fs_write,
    )
    return struct


def snapshot(
    auth,
    org,
    environments=["test", "prod"],
    prefix=None,
    fs_write=False,
    base_dir="snapshots",
):
    struct = Struct(
        apis={},
        kvms={},
        targetservers={},
        caches={},
        developers=[],
        apps={},
        products=[],
        roles=[],
    )
    snapshot_apis(
        auth, org, struct, environments=environments, prefix=prefix, fs_write=fs_write,
    )
    for env in environments:
        snapshot_kvms(
            auth, org, struct, environment=env, prefix=prefix, fs_write=fs_write,
        )
        snapshot_targetservers(
            auth, org, struct, environment=env, prefix=prefix, fs_write=fs_write,
        )
        snapshot_caches(
            auth, org, struct, environment=env, prefix=prefix, fs_write=fs_write,
        )
    snapshot_developers(
        auth, org, struct, prefix=prefix, fs_write=fs_write,
    )
    snapshot_products(
        auth, org, struct, prefix=prefix, fs_write=fs_write,
    )
    snapshot_apps(
        auth, org, struct, prefix=prefix, fs_write=fs_write,
    )
    snapshot_roles(
        auth, org, struct, prefix=prefix, fs_write=fs_write,
    )
    return struct


def backup_apis(auth, org, snapshot, prefix=None, fs_write=True, apis_dir="apis"):
    for api, metadata in snapshot.apis.items():
        for rev in metadata["revision"]:
            output_dir = str(Path(org) / apis_dir / api / rev)
            output_zip = str(Path(output_dir) / (api + ".zip"))
            touch(output_zip)
            export_proxy(auth, org, api, rev, fs_write=fs_write, output_file=output_zip)
            extractzip(output_zip, output_dir)
            os.remove(output_zip)
    return snapshot


def backup_kvms(
    auth,
    org,
    snapshot,
    environment,
    prefix=None,
    fs_write=True,
    kvms_dir="keyvaluemaps",
):
    for kvm in snapshot.kvms[environment]:
        write_file(
            get_kvm(auth, org, environment, kvm).text,
            str(Path(org) / kvms_dir / environment / (kvm + ".json")),
            fs_write=fs_write,
        )
    return snapshot


def backup_targetservers(
    auth,
    org,
    snapshot,
    environment,
    prefix=None,
    fs_write=True,
    targetservers_dir="targetservers",
):
    for targetserver in snapshot.targetservers[environment]:
        write_file(
            get_targetserver(auth, org, environment, targetserver).text,
            str(Path(org) / targetservers_dir / environment / (targetserver + ".json")),
            fs_write=fs_write,
        )
    return snapshot


def backup_caches(
    auth, org, snapshot, environment, prefix=None, fs_write=True, caches_dir="caches",
):
    for cache in snapshot.caches[environment]:
        write_file(
            get_cache(auth, org, environment, cache).text,
            str(Path(org) / caches_dir / environment / (cache + ".json")),
            fs_write=fs_write,
        )
    return snapshot


def backup_developers(
    auth, org, snapshot, prefix=None, fs_write=True, developers_dir="developers"
):
    for dev in snapshot.developers:
        write_file(
            get_developer(auth, org, dev).text,
            str(Path(org) / developers_dir / (dev + ".json")),
            fs_write=fs_write,
        )
    return snapshot


def backup_products(
    auth, org, snapshot, prefix=None, fs_write=True, products_dir="apiproducts"
):
    for product in snapshot.products:
        write_file(
            get_product(auth, org, product).text,
            str(Path(org) / products_dir / (product + ".json")),
            fs_write=fs_write,
        )
    return snapshot


def backup_apps(auth, org, snapshot, prefix=None, fs_write=True, apps_dir="apps"):
    for dev, apps in snapshot.apps.items():
        for app in apps:
            write_file(
                get_app(auth, org, dev, app).text,
                str(Path(org) / apps_dir / dev / (app + ".json")),
                fs_write=fs_write,
            )
    return snapshot


def backup_roles(
    auth, org, snapshot, prefix=None, fs_write=True, roles_dir="userroles"
):
    for role in snapshot.roles:
        write_file(
            get_users_for_role(auth, org, role),
            str(Path(org) / roles_dir / role / "users.json"),
            fs_write=fs_write,
        )
        write_file(
            get_permissions(auth, org, role),
            str(Path(org) / roles_dir / role / "resource_permissions.json"),
            fs_write=fs_write,
        )
    return snapshot


def backup(
    auth, org, environments=["test", "prod"], prefix=None, fs_write=True,
):
    curr_snapshot = snapshot(
        auth, org, environments=environments, prefix=prefix, fs_write=fs_write,
    )
    backup_apis(
        auth, org, curr_snapshot, prefix=prefix, fs_write=fs_write,
    )
    for env in environments:
        backup_kvms(
            auth, org, curr_snapshot, env, prefix=prefix, fs_write=fs_write,
        )
        backup_targetservers(
            auth, org, curr_snapshot, env, prefix=prefix, fs_write=fs_write,
        )
        backup_caches(
            auth, org, curr_snapshot, env, prefix=prefix, fs_write=fs_write,
        )
    backup_developers(auth, org, curr_snapshot, prefix=prefix, fs_write=fs_write)
    backup_products(auth, org, curr_snapshot, prefix=prefix, fs_write=fs_write)
    backup_apps(auth, org, curr_snapshot, prefix=prefix, fs_write=fs_write)
    backup_roles(auth, org, curr_snapshot, prefix=prefix, fs_write=fs_write)


def restore_kvms(auth, org, environment, directory, snapshot=[], dry_run=False):
    if not snapshot:
        return
    missing = []
    curr_snapshot = list_kvms(auth, org, environment)
    for kvm_file in os.listdir(directory):
        path = str(Path(directory) / kvm_file)
        kvm_name = read_file(path, type="json")["name"]
        if kvm_name in snapshot and kvm_name not in curr_snapshot:
            if not dry_run:
                Keyvaluemaps(auth, org, None).push_keyvaluemap(environment, path)
            missing.append(kvm_name)
    console.log(
        f"Missing key value maps that will be restored: {json.dumps(missing)}",
        silent=not dry_run,
    )
    return missing


def restore_targetservers(
    auth, org, environment, directory, snapshot=[], dry_run=False
):
    if not snapshot:
        return
    missing = []
    curr_snapshot = list_targetservers(auth, org, environment)
    for targetserver_file in os.listdir(directory):
        path = str(Path(directory) / targetserver_file)
        targetserver_name = read_file(path, type="json")["name"]
        if targetserver_name in snapshot and targetserver_name not in curr_snapshot:
            if not dry_run:
                Targetservers(auth, org, None).push_targetserver(environment, path)
            missing.append(targetserver_name)
    console.log(
        f"Missing target servers that will be restored: {json.dumps(missing)}",
        silent=not dry_run,
    )
    return missing


def restore_caches(auth, org, environment, directory, snapshot=[], dry_run=False):
    if not snapshot:
        return
    missing = []
    curr_snapshot = list_caches(auth, org, environment)
    for cache_file in os.listdir(directory):
        path = str(Path(directory) / cache_file)
        cache_name = read_file(path, type="json")["name"]
        if cache_name in snapshot and cache_name not in curr_snapshot:
            if not dry_run:
                Caches(auth, org, None).push_cache(environment, path)
            missing.append(cache_name)
    console.log(
        f"Missing caches that will be restored: {json.dumps(missing)}",
        silent=not dry_run,
    )
    return missing


def restore_developers(auth, org, directory, snapshot=[], dry_run=False):
    if not snapshot:
        return
    missing = []
    curr_snapshot = list_developers(auth, org)
    for developer_file in os.listdir(directory):
        path = str(Path(directory) / developer_file)
        developer_name = read_file(path, type="json")["email"]
        if developer_name in snapshot and developer_name not in curr_snapshot:
            if not dry_run:
                content = read_file(path, type="json")
                Developers(auth, org, content["email"]).create_developer(
                    content["firstName"],
                    content["lastName"],
                    content["userName"],
                    attributes=json.dumps({"attributes": content["attributes"]}),
                )
            missing.append(developer_name)
    console.log(
        f"Missing developers that will be restored: {json.dumps(missing)}",
        silent=not dry_run,
    )
    return missing


def restore_products(auth, org, directory, snapshot=[], dry_run=False):
    if not snapshot:
        return
    missing = []
    curr_snapshot = list_products(auth, org)
    for product_file in os.listdir(directory):
        path = str(Path(directory) / product_file)
        product_name = read_file(path, type="json")["name"]
        if product_name in snapshot and product_name not in curr_snapshot:
            if not dry_run:
                Apiproducts(auth, org, None).push_apiproducts(path)
            missing.append(product_name)
    console.log(
        f"Missing API products that will be restored: {json.dumps(missing)}",
        silent=not dry_run,
    )
    return missing


def restore_apps(auth, org, directory, snapshot_dir, dry_run=False):
    missing = {}
    curr_snapshot = list_apps(auth, org)
    pathlist = Path(directory).glob("**/*")
    for app_file in pathlist:
        if isfile(app_file):
            app_name = read_file(app_file, type="json")["name"]
            developer_name = os.path.split(os.path.split(app_file)[0])[1]
            try:
                snapshot = read_file(
                    str(Path(snapshot_dir) / (developer_name + ".json")), type="json",
                )
            except FileNotFoundError:
                snapshot = read_file(
                    str(Path(snapshot_dir) / developer_name), type="json"
                )
            try:
                curr_snapshot[developer_name]
            except KeyError:
                curr_snapshot[developer_name] = []
            if app_name in snapshot and app_name not in curr_snapshot[developer_name]:
                if not dry_run:
                    content = read_file(app_file, type="json")
                    content["developerId"] = developer_name
                    write_file(content, app_file)
                    Apps(auth, org, None).restore_app(app_file)
                try:
                    missing[developer_name].append(app_name)
                except KeyError:
                    missing[developer_name] = []
                    missing[developer_name].append(app_name)
    console.log(
        f"Missing developer apps that will be restored: {json.dumps(missing)}",
        silent=not dry_run,
    )
    return missing


def restore_permissions(auth, org, role_name, file):
    return Permissions(auth, org, role_name).create_permissions(
        json.dumps(read_file(file, type="json"))
    )


def restore_users(auth, org, role_name, file):
    users = read_file(file, type="json")
    for user in users:
        Userroles(auth, org, role_name).add_a_user_to_a_role(user)
    return get_users_for_role(auth, org, role_name)


def restore_roles(auth, org, directory, snapshot=[], dry_run=False):
    if not snapshot:
        return
    missing = []
    curr_snapshot = list_roles(auth, org)
    pathlist = Path(directory).glob("**/*")
    for role_path in pathlist:
        if isdir(role_path):
            role_name = os.path.split(role_path)[1]
            if role_name in snapshot and role_name not in curr_snapshot:
                if not dry_run:
                    Userroles(
                        auth, org, [role_name]
                    ).create_a_user_role_in_an_organization()
                    if isfile(str(Path(role_path) / "resource_permissions.json")):
                        restore_permissions(
                            auth,
                            org,
                            role_name,
                            str(Path(role_path) / "resource_permissions.json"),
                        )
                    if isfile(str(Path(role_path) / "users.json")):
                        restore_users(
                            auth, org, role_name, str(Path(role_path) / "users.json"),
                        )
                missing.append(role_name)
    console.log(
        f"Missing user roles that will be restored: {json.dumps(missing)}",
        silent=not dry_run,
    )
    return missing



import argparse

from apigee import APIGEE_ORG
from apigee import APIGEE_USERNAME
from apigee import APIGEE_PASSWORD
from apigee import APIGEE_MFA_SECRET
from apigee.util import *


class ParentParser:
    def __init__(self):
        self._parent_parser = argparse.ArgumentParser(add_help=False)
        self._build_profile_argument()
        self._profile = self._parent_parser.parse_known_args()[0].profile
        self._create_parser()

    @property
    def parent_parser(self):
        return self._parent_parser

    @parent_parser.setter
    def parent_parser(self, value):
        self._parent_parser = value

    @property
    def profile(self):
        return self._profile

    @profile.setter
    def profile(self, value):
        self._profile = value

    def __call__(self):
        return self._parent_parser

    def _build_profile_argument(self):
        self._parent_parser.add_argument(
            "-P",
            "--profile",
            action="store",
            help="name of credentials profile",
            default="default",
        )

    def _build_mfa_secret_argument(self):
        mfa_secret = authorization.get_credential(self._profile, "mfa_secret")
        self._parent_parser.add_argument(
            "--mfa-secret",
            action="store",
            help="apigee mfa secret",
            required=False,
            default=APIGEE_MFA_SECRET if mfa_secret is None else mfa_secret,
        )

    def _build_org_argument(self):
        org = authorization.get_credential(self._profile, "org")
        if org:
            self._parent_parser.add_argument(
                "-o",
                "--org",
                action="store",
                help="apigee org",
                required=False,
                default=org,
            )
        elif APIGEE_ORG:
            self._parent_parser.add_argument(
                "-o",
                "--org",
                action="store",
                help="apigee org",
                required=False,
                default=APIGEE_ORG,
            )
        else:
            self._parent_parser.add_argument(
                "-o", "--org", action="store", help="apigee org", required=True
            )

    def _build_username_argument(self):
        username = authorization.get_credential(self._profile, "username")
        if username:
            self._parent_parser.add_argument(
                "-u",
                "--username",
                action="store",
                help="apigee username",
                required=False,
                default=username,
            )
        elif APIGEE_USERNAME:
            self._parent_parser.add_argument(
                "-u",
                "--username",
                action="store",
                help="apigee username",
                required=False,
                default=APIGEE_USERNAME,
            )
        else:
            self._parent_parser.add_argument(
                "-u",
                "--username",
                action="store",
                help="apigee username",
                required=True,
            )

    def _build_password_argument(self):
        password = authorization.get_credential(self._profile, "password")
        if password:
            self._parent_parser.add_argument(
                "-p",
                "--password",
                action="store",
                help="apigee password",
                required=False,
                default=password,
            )
        elif APIGEE_PASSWORD:
            self._parent_parser.add_argument(
                "-p",
                "--password",
                action="store",
                help="apigee password",
                required=False,
                default=APIGEE_PASSWORD,
            )
        else:
            self._parent_parser.add_argument(
                "-p",
                "--password",
                action="store",
                help="apigee password",
                required=True,
            )

    def _create_parser(self):
        self._build_mfa_secret_argument()
        self._build_org_argument()
        self._build_username_argument()
        self._build_password_argument()



#!/usr/bin/env python
"""Source: https://apidocs.apigee.com/api-services/content/environment-keyvalue-maps

The Key/Value maps API lets you create and manage collections of arbitrary
key/value pairs at the environment scope (for example, test or prod) for
longer-term data persistence.
"""

import json
import requests
from requests.exceptions import HTTPError

from tqdm import tqdm

from apigee import APIGEE_ADMIN_API_URL
from apigee.abstract.api.keyvaluemaps import IKeyvaluemaps, KeyvaluemapsSerializer
from apigee.util import authorization, console


class Keyvaluemaps(IKeyvaluemaps):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def create_keyvaluemap_in_an_environment(self, environment, request_body):
        """Creates a key value map in an environment.

        A key value map is a simple structure for persistently storing
        name/value pairs as entries in a named map.
        The entries in a KVM can be retrieved at runtime by the Key Value Map
        Operations policy or code running on Apigee Edge.
        Use KVMs for use cases such as profile-based access control, storing
        environment-specific data, to control application-specific behavior,
        and so on.

        You can created an encrypted KVM by adding "encrypted" : "true" to the
        payload.

        KVM names are case sensitive.

        Args:
            environment (str): Apigee environment.
            request_body (str): JSON string.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/keyvaluemaps"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        body = json.loads(request_body)
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def delete_keyvaluemap_from_an_environment(self, environment):
        """Deletes a key/value map and all associated entries from an
        environment.

        Args:
            environment (str): Apigee environment.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/keyvaluemaps/{self._map_name}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.delete(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def delete_keyvaluemap_entry_in_an_environment(self, environment, entry_name):
        """Deletes a specific key/value map entry in an environment by name,
        along with associated entries.

        Args:
            environment (str): Apigee environment.
            entry_name (str): The entry to delete.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/keyvaluemaps/{self._map_name}/entries/{entry_name}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.delete(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def get_keyvaluemap_in_an_environment(self, environment):
        """Gets a KeyValueMap (KVM) in an environment by name, along with the
        keys and values.

        KVM names are case sensitive.

        With Apigee Edge for Public Cloud, this API returns only the first 100
        keys in a KVM.

        Args:
            environment (str): Apigee environment.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/keyvaluemaps/{self._map_name}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def get_a_keys_value_in_an_environment_scoped_keyvaluemap(
        self, environment, entry_name
    ):
        """Gets the value of a key in an environment-scoped KeyValueMap (KVM).

        Args:
            environment (str): Apigee environment.
            entry_name (str): The entry name.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/keyvaluemaps/{self._map_name}/entries/{entry_name}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def list_keyvaluemaps_in_an_environment(self, environment, prefix=None):
        """Lists the name of all key/value maps in an environment.

        Optionally returns an expanded view of all key/value maps for the
        environment.

        Args:
            environment (str): Apigee environment.
            prefix (str, optional): Filter results by a prefix string.
                Defaults to None.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/keyvaluemaps"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return KeyvaluemapsSerializer().serialize_details(resp, "json", prefix=prefix)

    def update_keyvaluemap_in_an_environment(self, environment, request_body):
        """Updates an existing KeyValueMap in an environment.

        Note: This API is supported for Apigee Edge for Private Cloud only.
        For Apigee Edge for Public Cloud use Update an entry in an
        environment-scoped KVM.

        Does not override the existing map. Instead, this method updates the
        entries if they exist or adds them if not.

        It can take several minutes before the new value is visible to runtime
        traffic.

        Args:
            environment (str): Apigee environment.
            request_body (str): JSON string.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/keyvaluemaps/{self._map_name}"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        body = json.loads(request_body)
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def create_an_entry_in_an_environment_scoped_kvm(
        self, environment, entry_name, entry_value
    ):
        """Creates an entry in an existing KeyValueMap scoped to an environment.

        Note: This API is supported for Apigee Edge for the Public Cloud only.

        A key (name) cannot be larger than 2 KB. KVM names are case sensitive.

        Args:
            environment (str): Apigee environment.
            entry_name (str): The entry name to create.
            entry_value (str): The entry value to create.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/keyvaluemaps/{self._map_name}/entries"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        body = {"name": entry_name, "value": entry_value}
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def update_an_entry_in_an_environment_scoped_kvm(
        self, environment, entry_name, updated_value
    ):
        """Updates an entry in a KeyValueMap scoped to an environment.

        Note: This API is supported for Apigee Edge for the Public Cloud only.

        A key cannot be larger than 2 KB. KVM names are case sensitive.

        Does not override the existing map.
        Instead, this method updates the entries if they exist or adds them if
        not.
        It can take several minutes before the new value is visible to runtime
        traffic.

        Args:
            environment (str): Apigee environment.
            entry_name (str): The entry name to update.
            updated_value (str): The new entry value.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/keyvaluemaps/{self._map_name}/entries/{entry_name}"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        body = {"name": entry_name, "value": updated_value}
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def list_keys_in_an_environment_scoped_keyvaluemap(
        self, environment, startkey, count
    ):
        """Lists keys in a KeyValueMap scoped to an environment.

        Note: This API is supported for Apigee Edge for the Public Cloud only.

        Args:
            environment (str): Apigee environment.
            startkey (str): To filter the keys that are returned, enter the name
                of a key that the list will start with.
            count (int): Limits the list of keys to the number you specify, up
                to a maximum of 100.
                Use with the startkey parameter to provide more targeted
                filtering.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/keyvaluemaps/{self._map_name}/keys?startkey={startkey}&count={count}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        # return KeyvaluemapsSerializer().serialize_details(resp, 'json', prefix=prefix)
        return resp

    def _diff_kvms(self, kvm1, kvm2):
        current_keys = [entry["name"] for entry in kvm1["entry"]]
        deleted_keys = [
            entry for entry in kvm2["entry"] if entry["name"] not in current_keys
        ]
        return current_keys, deleted_keys

    def _create_or_update_entry(self, environment, entry):
        try:
            self.get_a_keys_value_in_an_environment_scoped_keyvaluemap(
                environment, entry["name"]
            )
            self.update_an_entry_in_an_environment_scoped_kvm(
                environment, entry["name"], entry["value"]
            )
        except HTTPError as e:
            if e.response.status_code not in [404]:
                raise e
            self.create_an_entry_in_an_environment_scoped_kvm(
                environment, entry["name"], entry["value"]
            )

    def _delete_entries(self, environment, deleted_keys):
        for idx, entry in enumerate(tqdm(deleted_keys)):
            self.delete_keyvaluemap_entry_in_an_environment(environment, entry["name"])

    def push_keyvaluemap(self, environment, file):
        """Push KeyValueMap file to Apigee

        This will create a KeyValueMap if it does not exist and update if it
        does.

        Args:
            environment (str): Apigee environment.
            file (str): The file path.

        Returns:
            None

        Raises:
            HTTPError: If response status code is not successful or 404.
        """
        with open(file) as f:
            body = f.read()
        loc_kvm = json.loads(body)
        self._map_name = loc_kvm["name"]
        try:
            env_kvm = self.get_keyvaluemap_in_an_environment(environment).json()
            _, deleted_keys = self._diff_kvms(loc_kvm, env_kvm)
            console.log("Updating entries in", self._map_name)
            for idx, entry in enumerate(tqdm(loc_kvm["entry"])):
                if entry not in env_kvm["entry"]:
                    self._create_or_update_entry(environment, entry)
            if deleted_keys:
                console.log("Deleting entries in", self._map_name)
                self._delete_entries(environment, deleted_keys)
        except HTTPError as e:
            if e.response.status_code not in [404]:
                raise e
            console.log("Creating", self._map_name)
            console.log(
                self.create_keyvaluemap_in_an_environment(environment, body).text
            )



#!/usr/bin/env python
"""https://apidocs.apigee.com/api/caches"""

import json
from abc import ABC, abstractmethod


class ICaches:
    def __init__(self, auth, org_name, cache_name):
        self._auth = auth
        self._org_name = org_name
        self._cache_name = cache_name

    def __call__(self):
        pass

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @property
    def cache_name(self):
        return self._cache_name

    @cache_name.setter
    def cache_name(self, value):
        self._cache_name = value

    @abstractmethod
    def clear_all_cache_entries(self, environment):
        pass

    @abstractmethod
    def clear_a_cache_entry(self, environment, entry):
        pass

    @abstractmethod
    def create_a_cache_in_an_environment(self, environment, request_body):
        pass

    @abstractmethod
    def get_information_about_a_cache(self, environment):
        pass

    @abstractmethod
    def list_caches_in_an_environment(self, environment, prefix=None):
        pass

    @abstractmethod
    def update_a_cache_in_an_environment(self, environment, request_body):
        pass

    @abstractmethod
    def delete_a_cache(self, environment):
        pass

    @abstractmethod
    def push_cache(self, environment, file):
        pass


class CachesSerializer:
    def serialize_details(self, caches, format, prefix=None):
        resp = caches
        if format == "text":
            return caches.text
        caches = caches.json()
        if prefix:
            caches = [cache for cache in caches if cache.startswith(prefix)]
        if format == "json":
            return json.dumps(caches)
        elif format == "table":
            pass
        # else:
        #     raise ValueError(format)
        return resp



import argparse


class FormatParser:
    def __init__(self):
        self._parent_parser = argparse.ArgumentParser(add_help=False)
        self._create_parser()

    @property
    def parent_parser(self):
        return self._parent_parser

    @parent_parser.setter
    def parent_parser(self, value):
        self._parent_parser = value

    def __call__(self):
        return self._parent_parser

    def _build_format_argument(self):
        self._parent_parser.add_argument(
            "-F", "--format", action="store", help="output format type", required=False
        )

    def _create_parser(self):
        self._build_format_argument()



APP = "apigeecli"
CMD = "apigee"
__version__ = "0.34.0"
description = (
    "Apigee Management API command-line interface with multi-factor authentication"
)

import json
from os import getenv
from pathlib import Path

APIGEE_CLI_AUTHORIZATION_DEVELOPER_ATTRIBUTE = "team"

APIGEE_CLI_DIRECTORY = str(Path.home() / ".apigee")
APIGEE_CLI_ACCESS_TOKEN_FILE = str(Path(APIGEE_CLI_DIRECTORY) / "access_token")
APIGEE_CLI_CREDENTIALS_FILE = str(Path(APIGEE_CLI_DIRECTORY) / "credentials")
APIGEE_CLI_EXCEPTION_LOG_FILE = str(Path(APIGEE_CLI_DIRECTORY) / "exception.log")

APIGEE_CLI_PREFIX = getenv("APIGEE_CLI_PREFIX")
APIGEE_USERNAME = getenv("APIGEE_USERNAME")
APIGEE_PASSWORD = getenv("APIGEE_PASSWORD")
APIGEE_MFA_SECRET = getenv("APIGEE_MFA_SECRET")
APIGEE_ADMIN_API_URL = "https://api.enterprise.apigee.com"
APIGEE_OAUTH_URL = "https://login.apigee.com/oauth/token"
APIGEE_ORG = getenv("APIGEE_ORG")

APIGEE_CLI_TOGGLE_SILENT = False
APIGEE_CLI_TOGGLE_VERBOSE = 0



#!/usr/bin/env python
"""https://apidocs.apigee.com/api/api_resources/51-targetservers"""

import json
from abc import ABC, abstractmethod


class ITargetservers:
    def __init__(self, auth, org_name, targetserver_name):
        self._auth = auth
        self._org_name = org_name
        self._targetserver_name = targetserver_name

    def __call__(self):
        pass

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @property
    def targetserver_name(self):
        return self._targetserver_name

    @targetserver_name.setter
    def targetserver_name(self, value):
        self._targetserver_name = value

    @abstractmethod
    def create_a_targetserver(self, environment, request_body):
        pass

    @abstractmethod
    def delete_a_targetserver(self, environment):
        pass

    @abstractmethod
    def list_targetservers_in_an_environment(self, environment, prefix=None):
        pass

    @abstractmethod
    def get_targetserver(self, environment):
        pass

    @abstractmethod
    def update_a_targetserver(self, environment, request_body):
        pass

    @abstractmethod
    def push_targetserver(self, environment, file):
        pass


class TargetserversSerializer:
    def serialize_details(self, targetservers, format, prefix=None):
        resp = targetservers
        if format == "text":
            return targetservers.text
        targetservers = targetservers.json()
        if prefix:
            targetservers = [
                targetserver
                for targetserver in targetservers
                if targetserver.startswith(prefix)
            ]
        if format == "json":
            return json.dumps(targetservers)
        elif format == "table":
            pass
        # else:
        #     raise ValueError(format)
        return resp



#!/usr/bin/env python
"""Source: https://docs.apigee.com/api-platform/system-administration/permissions

Manage permissions that you can assign to a role by using the Edge API.
"""

import json
import requests

from apigee import APIGEE_ADMIN_API_URL
from apigee.abstract.api.permissions import IPermissions, PermissionsSerializer
from apigee.util import authorization


class Permissions(IPermissions):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def create_permissions(self, request_body):
        """Creates permissions for a role.

        Args:
            request_body (str): JSON string.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/userroles/{self._role_name}/resourcepermissions"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        body = json.loads(request_body)
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def team_permissions(
        self, template_file, placeholder_key=None, placeholder_value=""
    ):
        """Creates permissions for a role using a template file.

        Args:
            template_file (str): The template file path.
            placeholder_key (str, optional): The placeholder key to replace with
                a placeholder value.
                Defaults to None.
            placeholder_value (str, optional): The placeholder value to replace
                placeholder key.
                Default to ''.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/userroles/{self._role_name}/resourcepermissions"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        with open(template_file) as f:
            body = json.loads(f.read())
        if placeholder_key:
            for idx, resource_permission in enumerate(body["resourcePermission"]):
                path = resource_permission["path"]
                body["resourcePermission"][idx]["path"] = path.replace(
                    placeholder_key, placeholder_value
                )
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def get_permissions(
        self, formatted=False, format="text", showindex=False, tablefmt="plain"
    ):
        """Gets permissions for a role.

        Args:
            formatted (bool, optional): If True, format ``requests.Response()``.
                Defaults to False.
            format (str, optional): Specify how to format response.
                Defaults to 'text'.
            showindex (bool, optional): If True, show table index.
                Defaults to False.
            tablefmt (str, optional): Specify table format. Defaults to 'plain'.

        Returns:
            requests.Response(): Response if ``formatted`` is False, else return
            a ``formatted`` value.
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/o/{self._org_name}/userroles/{self._role_name}/permissions"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        if formatted:
            return PermissionsSerializer().serialize_details(
                resp, format, showindex=showindex, tablefmt=tablefmt
            )
        return resp



import argparse

from apigee.api.backup_and_restore import backup

from apigee.parsers.parent_parser import ParentParser
from apigee.parsers.environment_parser import EnvironmentParser
from apigee.parsers.prefix_parser import PrefixParser
from apigee.parsers.silent_parser import SilentParser
from apigee.parsers.verbose_parser import VerboseParser

from apigee.util import console
from apigee.util.authorization import gen_auth


class ParserBackup:
    def __init__(self, parser, **kwargs):
        self._parser = parser
        self._parent_parser = kwargs.get("parent_parser", ParentParser())
        self._environment_parser = kwargs.get("environment_parser", EnvironmentParser())
        self._prefix_parser = kwargs.get(
            "prefix_parser", PrefixParser(profile="default")
        )
        self._silent_parser = kwargs.get("silent_parser", SilentParser())
        self._verbose_parser = kwargs.get("verbose_parser", VerboseParser())
        self._parser_backup = self._parser.add_parser(
            "backup",
            help="admin backup",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        self._create_parser()

    @property
    def parser(self):
        return self._parser

    @parser.setter
    def parser(self, value):
        self._parser = value

    @property
    def parser_backup(self):
        return self._parser_backup

    @parser_backup.setter
    def parser_backup(self, value):
        self._parser_backup = value

    @property
    def parent_parser(self):
        return self._parent_parser

    @parent_parser.setter
    def parent_parser(self, value):
        self._parent_parser = value

    @property
    def environment_parser(self):
        return self._environment_parser

    @environment_parser.setter
    def environment_parser(self, value):
        self._environment_parser = value

    @property
    def prefix_parser(self):
        return self._prefix_parser

    @prefix_parser.setter
    def prefix_parser(self, value):
        self._prefix_parser = value

    def __call__(self):
        return self._parser

    def _build_backup_argument(self):
        self._parser_backup.add_argument(
            "--environments", nargs="+", help="list of environments", required=True
        )
        self._parser_backup.set_defaults(
            func=lambda args: backup(
                gen_auth(args.username, args.password, args.mfa_secret),
                args.org,
                environments=args.environments,
                fs_write=True,
            )
        )

    def _create_parser(self):
        self._build_backup_argument()



import argparse

from apigee.api import backup_and_restore as bar

from apigee.parsers.parent_parser import ParentParser
from apigee.parsers.environment_parser import EnvironmentParser
from apigee.parsers.prefix_parser import PrefixParser
from apigee.parsers.silent_parser import SilentParser
from apigee.parsers.verbose_parser import VerboseParser

from apigee.util import console
from apigee.util.authorization import gen_auth
from apigee.util.os import read_file


class ParserRestore:
    def __init__(self, parser, **kwargs):
        self._parser = parser
        self._parent_parser = kwargs.get("parent_parser", ParentParser())
        self._environment_parser = kwargs.get("environment_parser", EnvironmentParser())
        self._prefix_parser = kwargs.get(
            "prefix_parser", PrefixParser(profile="default")
        )
        self._silent_parser = kwargs.get("silent_parser", SilentParser())
        self._verbose_parser = kwargs.get("verbose_parser", VerboseParser())
        self._parser_restore = self._parser.add_parser(
            "restore", help="restore specific resources from backup",
        ).add_subparsers()
        self._create_parser()

    @property
    def parser(self):
        return self._parser

    @parser.setter
    def parser(self, value):
        self._parser = value

    @property
    def parser_restore(self):
        return self._parser_restore

    @parser_restore.setter
    def parser_restore(self, value):
        self._parser_restore = value

    @property
    def parent_parser(self):
        return self._parent_parser

    @parent_parser.setter
    def parent_parser(self, value):
        self._parent_parser = value

    @property
    def environment_parser(self):
        return self._environment_parser

    @environment_parser.setter
    def environment_parser(self, value):
        self._environment_parser = value

    @property
    def prefix_parser(self):
        return self._prefix_parser

    @prefix_parser.setter
    def prefix_parser(self, value):
        self._prefix_parser = value

    def __call__(self):
        return self._parser

    def _build_restore_kvms_argument(self):
        parser = self._parser_restore.add_parser(
            "keyvaluemaps",
            aliases=["kvms"],
            help="",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.add_argument(
            "--environments", help="list of environments", required=True
        )
        parser.add_argument(
            "-d", "--directory", help="directory of key value maps", required=True
        )
        parser.add_argument(
            "--snapshot-file", help="key value maps snapshot file", required=True
        )
        parser.add_argument(
            "--dry-run",
            action="store_true",
            help="check which key value maps will be restored",
        )
        parser.set_defaults(
            func=lambda args: bar.restore_kvms(
                gen_auth(args.username, args.password, args.mfa_secret),
                args.org,
                args.environments,
                args.directory,
                snapshot=read_file(args.snapshot_file),
                dry_run=args.dry_run,
            )
        )

    def _build_restore_targetservers_argument(self):
        parser = self._parser_restore.add_parser(
            "targetservers",
            aliases=["ts"],
            help="",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.add_argument("--environment", help="list of environments", required=True)
        parser.add_argument(
            "-d", "--directory", help="directory of target servers", required=True
        )
        parser.add_argument(
            "--snapshot-file", help="target servers snapshot file", required=True
        )
        parser.add_argument(
            "--dry-run",
            action="store_true",
            help="check which target servers will be restored",
        )
        parser.set_defaults(
            func=lambda args: bar.restore_targetservers(
                gen_auth(args.username, args.password, args.mfa_secret),
                args.org,
                args.environment,
                args.directory,
                snapshot=read_file(args.snapshot_file),
                dry_run=args.dry_run,
            )
        )

    def _build_restore_caches_argument(self):
        parser = self._parser_restore.add_parser(
            "caches",
            help="",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.add_argument("--environment", help="list of environments", required=True)
        parser.add_argument(
            "-d", "--directory", help="directory of caches", required=True
        )
        parser.add_argument(
            "--snapshot-file", help="caches snapshot file", required=True
        )
        parser.add_argument(
            "--dry-run",
            action="store_true",
            help="check which caches will be restored",
        )
        parser.set_defaults(
            func=lambda args: bar.restore_caches(
                gen_auth(args.username, args.password, args.mfa_secret),
                args.org,
                args.environment,
                args.directory,
                snapshot=read_file(args.snapshot_file),
                dry_run=args.dry_run,
            )
        )

    def _build_restore_developers_argument(self):
        parser = self._parser_restore.add_parser(
            "developers",
            aliases=["devs"],
            help="",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.add_argument(
            "-d", "--directory", help="directory of developers", required=True
        )
        parser.add_argument(
            "--snapshot-file", help="developers snapshot file", required=True
        )
        parser.add_argument(
            "--dry-run",
            action="store_true",
            help="check which developers will be restored",
        )
        parser.set_defaults(
            func=lambda args: bar.restore_developers(
                gen_auth(args.username, args.password, args.mfa_secret),
                args.org,
                args.directory,
                snapshot=read_file(args.snapshot_file),
                dry_run=args.dry_run,
            )
        )

    def _build_restore_products_argument(self):
        parser = self._parser_restore.add_parser(
            "apiproducts",
            aliases=["products"],
            help="",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.add_argument(
            "-d", "--directory", help="directory of API products", required=True
        )
        parser.add_argument(
            "--snapshot-file", help="API products snapshot file", required=True
        )
        parser.add_argument(
            "--dry-run",
            action="store_true",
            help="check which API products will be restored",
        )
        parser.set_defaults(
            func=lambda args: bar.restore_products(
                gen_auth(args.username, args.password, args.mfa_secret),
                args.org,
                args.directory,
                snapshot=read_file(args.snapshot_file),
                dry_run=args.dry_run,
            )
        )

    def _build_restore_apps_argument(self):
        parser = self._parser_restore.add_parser(
            "apps",
            help="",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.add_argument(
            "-d", "--directory", help="directory of developer apps", required=True
        )
        parser.add_argument(
            "--snapshot-dir", help="developer apps snapshot directory", required=True
        )
        parser.add_argument(
            "--dry-run",
            action="store_true",
            help="check which developer apps will be restored",
        )
        parser.set_defaults(
            func=lambda args: bar.restore_apps(
                gen_auth(args.username, args.password, args.mfa_secret),
                args.org,
                args.directory,
                args.snapshot_dir,
                dry_run=args.dry_run,
            )
        )

    def _build_restore_roles_argument(self):
        parser = self._parser_restore.add_parser(
            "userroles",
            aliases=["roles"],
            help="",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.add_argument(
            "-d", "--directory", help="directory of developer apps", required=True
        )
        parser.add_argument(
            "--snapshot-file", help="developer apps snapshot file", required=True
        )
        parser.add_argument(
            "--dry-run",
            action="store_true",
            help="check which developer apps will be restored",
        )
        parser.set_defaults(
            func=lambda args: bar.restore_roles(
                gen_auth(args.username, args.password, args.mfa_secret),
                args.org,
                args.directory,
                snapshot=read_file(args.snapshot_file),
                dry_run=args.dry_run,
            )
        )

    def _create_parser(self):
        self._build_restore_kvms_argument()
        self._build_restore_targetservers_argument()
        self._build_restore_caches_argument()
        self._build_restore_developers_argument()
        self._build_restore_products_argument()
        self._build_restore_apps_argument()
        self._build_restore_roles_argument()



#!/usr/bin/env python
"""Source: https://apidocs.apigee.com/api/api-products-1

Base Path: https://api.enterprise.apigee.com/v1/organizations/{org_name}

API Resource Path: /apiproducts

A list of URIs with an associated Quota (or "service plan"), which is used to
expose customized API bundles to different developer groups.

API products enable you to bundle and distribute your APIs to multiple developer
groups simultaneously, without having to modify code. An API product consists of
a list of API resources (URIs) combined with a Service Plan (rate-limiting
policy settings) plus any custom metadata required by the API provider.
API products provide the basis for access control in Apigee, since they provide
control over the set of API resources that apps are allowed to consume.

As part of the app provisioning workflow, developers select from a list of
API products. This selection of an API product is usually made in the context of
a developer portal. The developer app is provisioned with a key and secret
(generated by and stored on Apigee Edge) that enable the app to access the URIs
bundled in the selected API product. To access access API resources bundled in
an API product, the app must present the API key issued by Apigee Edge.
Apigee Edge will resolve the key that is presented against an API product, and
then check associated  API resources and quota settings.

The API supports multiple API products per app key--your developers can consume
multiple API products without requiring multiple keys. Also, a key can be
'promoted' from one API product to another. This enables you to promote
developers from 'free' to 'premium' API products seamlessly and without user
interruption.
"""

import json
import requests
from requests.exceptions import HTTPError

from apigee import APIGEE_ADMIN_API_URL
from apigee.abstract.api.apiproducts import IApiproducts, ApiproductsSerializer
from apigee.util import authorization, console


class Apiproducts(IApiproducts):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def create_api_product(self, request_body):
        """Create an API product

        A list of API resources (URIs) combined with Quota settings that you can
        use to deliver customized API bundles to your developers.

        Args:
            request_body (str): JSON string.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/apiproducts"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        body = json.loads(request_body)
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def delete_api_product(self):
        """Deletes an API Product

        Args:
            None

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/apiproducts/{self._apiproduct_name}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.delete(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def get_api_product(self):
        """Gets an API Product

        Args:
            None

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/apiproducts/{self._apiproduct_name}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def list_api_products(self, prefix=None, expand=False, count=1000, startkey=""):
        """Lists all API Products by name for an organization

        Args:
            prefix (str, optional): Filter results by a prefix string.
                Defaults to None.
            expand (bool, optional): If True, show product details.
                Defaults to False.
            count (int, optional): Apigee Edge for Public Cloud only:
                Number of API products to return in the API call.
                The maximum limit is 1000. Use with the startkey to provide more
                targeted filtering.
            startkey (str, optional): Apigee Edge for Public Cloud only:
                Returns a list of API products starting with the specified
                API product.
                For example, if you're returning 50 API products at a time
                (using the count query parameter), you can view products 50-99
                by entering the name of the 50th API product in the first API.
                The API product name is case sensitive.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/apiproducts?expand={expand}&count={count}&startKey={startkey}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return ApiproductsSerializer().serialize_details(resp, "json", prefix=prefix)

    def update_api_product(self, request_body):
        """This method updates an existing API product

        Args:
            request_body (str): JSON string.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/apiproducts/{self._apiproduct_name}"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        body = json.loads(request_body)
        resp = requests.put(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def push_apiproducts(self, file):
        """Push API product file to Apigee

        This will create an API product if it does not exist and update if it
        does.

        Args:
            file (str): The file path.

        Returns:
            None

        Raises:
            HTTPError: If response status code is not successful or 404.
        """
        with open(file) as f:
            body = f.read()
        apiproduct = json.loads(body)
        self._apiproduct_name = apiproduct["name"]
        try:
            self.get_api_product()
            console.log("Updating", self._apiproduct_name)
            console.log(self.update_api_product(body).text)
        except HTTPError as e:
            if e.response.status_code not in [404]:
                raise e
            console.log("Creating", self._apiproduct_name)
            console.log(self.create_api_product(body).text)



#!/usr/bin/env python
"""Source: https://apidocs.apigee.com/api/apps-developer,
https://apidocs.apigee.com/api/developer-app-keys

Apps associated with developers (as developer apps).

Apps are API consumers registered with an API provider's organization.
Apps are registered with an organization to obtain credentials that enable
access to one or more API products (or, a set of URIs). The default app profile
can be extended by using custom attributes.
"""

import json
import random
import requests
import string

from apigee import APIGEE_ADMIN_API_URL
from apigee.abstract.api.apps import IApps, AppsSerializer
from apigee.util import authorization, console


class Apps(IApps):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def create_developer_app(self, developer, request_body):
        """Creates an app associated with a developer, associates the app with
        an API product, and auto-generates an API key for the app to use in
        calls to API proxies inside the API product.

        The name is the unique ID of the app that you can use in management API
        calls.
        The DisplayName (set with an attribute) is what appears in the
        management UI.
        If you don't provide a DisplayName, the name is used.

        The keyExpiresIn property sets the expiration on the API key.
        If you don't set this, or set the value to -1, they API key never
        expires.

        Apigee recommends that you use the developer's email address when
        calling this API.
        Developer IDs are generated internally by Apigee and are not guaranteed
        to stay the same over time.

        Args:
            developer (str): The developer's email address or ID.
            request_body (str): JSON string.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/developers/{developer}/apps"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        body = json.loads(request_body)
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def delete_developer_app(self, developer):
        """Deletes a developer app.

        Note the following:
          - With Apigee Edge for Public Cloud, deletion of the developer app and
            associated artifacts happens asynchronously.
            The developer app is deleted immediately, but the resources
            associated with that developer app, such as app keys or access
            tokens, may take anywhere from a few seconds to a few minutes to be
            automatically deleted.
          - Apigee recommends that you use the developer's email address when
            calling this API. Developer IDs are generated internally by Apigee
            and are not guaranteed to stay the same over time.

        Args:
            developer (str): The developer's email address or ID.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/developers/{developer}/apps/{self._app_name}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.delete(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def create_empty_developer_app(self, developer, display_name="", callback_url=""):
        """Creates an empty developer app.

        This method combines three API calls to Apigee:
          1. Create a developer app with the option to specify display name and
             callback URL
          2. Delete default key created for the new developer app
          3. Get the developer details

        Args:
            developer (str): The developer's email address or ID.
            display_name (str, optional): The ``DisplayName`` is what appears in
                the management UI. If you don't provide a ``DisplayName``, the
                ``name`` is used.
            callback_url (str, optional): The callbackUrl is used by OAuth 2.0
                authorization servers to communicate authorization codes back to
                apps.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/developers/{developer}/apps"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        body = {
            "name": self._app_name,
            # "apiProducts": args.products,
            "attributes": [{"name": "DisplayName", "value": display_name}],
            # "scopes" : args.scopes,
            "callbackUrl": callback_url,
        }
        if not display_name:
            del body["attributes"]
        if not callback_url:
            del body["callbackUrl"]
        # body = {k: v for k, v in body.items() if v}
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        self.delete_key_for_a_developer_app(
            developer, resp.json()["credentials"][0]["consumerKey"]
        )
        return self.get_developer_app_details(developer)

    def get_developer_app_details(self, developer):
        """Get the profile of a specific developer app.

        All times in the response are UNIX times.

        Note that the response contains a top-level attribute named accessType
        that is no longer used by Apigee.

        Args:
            developer (str): The developer's email address or ID.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/developers/{developer}/apps/{self._app_name}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def list_developer_apps(
        self, developer, prefix=None, expand=False, count=100, startkey=""
    ):
        """Lists all apps created by a developer in an organization, and
        optionally provides an expanded view of the apps.

        All time values in the response are UNIX times.

        You can specify either the developer's email address or Edge ID.

        Args:
            developer (str): The developer's email address or ID.
            prefix (str, optional): Filter results by a prefix string.
                Defaults to None.
            expand (bool, optional): If True, show app details.
                Defaults to False.
            count (int, optional): This parameter can be used with Apigee Edge
                for Public Cloud only.
                Limits the list to the number you specify. The limit is 100.
                Use with the startKey parameter to provide more targeted
                filtering.
            startkey (str, optional): This parameter can be used with Apigee
                Edge for Public Cloud only.
                To filter the keys that are returned, enter the name of a
                company app that the list will start with.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/developers/{developer}/apps"
        if expand:
            uri += f"?expand={expand}"
        else:
            uri += f"?count={count}&startKey={startkey}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return AppsSerializer().serialize_details(resp, "json", prefix=prefix)

    def delete_key_for_a_developer_app(self, developer, consumer_key):
        """Deletes a consumer key that belongs to an app, and removes all API
        products associated with the app.

        Once deleted, the consumer key cannot be used to access any APIs.

        Args:
            developer (str): The developer's email address or ID.
            consumer_key (str): Consumer key to delete.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/developers/{developer}/apps/{self._app_name}/keys/{consumer_key}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.delete(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def create_a_consumer_key_and_secret(
        self,
        developer,
        consumer_key=None,
        consumer_secret=None,
        key_length=32,
        secret_length=32,
        key_suffix=None,
        key_delimiter="-",
        products=[],
    ):
        """Creates a custom consumer key and secret for a developer app.

        This is particularly useful if you want to migrate existing consumer
        keys/secrets to Edge from another system.

        Consumer keys and secrets can contain letters, numbers, underscores,
        and hyphens. No other special characters are allowed.

        Args:
            developer (str): The developer's email address or ID.
            consumer_key (str, optional): Consumer key to create.
            consumer_secret (str, optional): Consumer secret to create.
            key_length (int, optional): Length of the consumer key to generate
                if ``consumer_key`` is not specified.
                Defaults to 32 characters.
            secret_length (int, optional): Length of the consumer secret to
                generate if ``consumer_secret`` is not specified.
                Defaults to 32 characters.
            key_suffix (str, optional): String to append to consumer key.
                Defaults to None.
            key_delimiter (str, optional): String to delimit ``consumer_key``
                and ``key_suffix``.
                Defaults to '-'.
            products (list, optional): List of API Products to add to the
                consumer key and secret.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/developers/{developer}/apps/{self._app_name}/keys/create"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        # app = self.get_developer_app_details(developer)
        if not consumer_key:
            consumer_key = "".join(
                random.SystemRandom().choice(string.ascii_letters + string.digits)
                for _ in range(key_length)
            )
        if not consumer_secret:
            consumer_secret = "".join(
                random.SystemRandom().choice(string.ascii_letters + string.digits)
                for _ in range(secret_length)
            )
        if key_suffix:
            consumer_key += key_delimiter
            consumer_key += key_suffix
        body = {"consumerKey": consumer_key, "consumerSecret": consumer_secret}
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        if products:
            console.log(resp.text)
            consumer_key = resp.json()["consumerKey"]
            request_body = json.dumps(
                {"apiProducts": products, "attributes": resp.json()["attributes"]}
            )
            console.log("Adding API Products", products, "to consumerKey", consumer_key)
            return self.add_api_product_to_key(developer, consumer_key, request_body)
        return resp

    def add_api_product_to_key(self, developer, consumer_key, request_body):
        """Adds an API product to a developer app key, enabling the app that
        holds the key to access the API resources bundled in the API product.

        You can also use this API to add attributes to the key.

        Args:
            developer (str): The developer's email address or ID.
            consumer_key (str, optional): Consumer key to add the API Product
                to.
            request_body (str): JSON string.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/developers/{developer}/apps/{self._app_name}/keys/{consumer_key}"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        body = json.loads(request_body)
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def restore_app(self, file):
        """Restore developer app file to Apigee.

        This method combines four different types of API calls to Apigee:
          1. Create a developer app
          2. Delete default key created for the new developer app
          3. For each consumer key and secret pair, add (create) to the new app
          4. Get the developer details

        Args:
            file (str): The file path.

        Returns:
            requests.Response()
        """
        with open(file, "r") as f:
            app = json.loads(f.read())
        self._app_name = app["name"]
        request_body = {}
        request_body["name"] = app["name"]
        request_body["attributes"] = app.get("attributes")
        request_body["scopes"] = app.get("scopes")
        request_body["callbackUrl"] = app.get("callbackUrl")
        request_body = {k: v for k, v in request_body.items() if v}
        resp = self.create_developer_app(app["developerId"], json.dumps(request_body))
        consumer_key = resp.json()["credentials"][0]["consumerKey"]
        self.delete_key_for_a_developer_app(app["developerId"], consumer_key)
        if app["credentials"]:
            for cred in app["credentials"]:
                consumer_key = cred["consumerKey"]
                consumer_secret = cred["consumerSecret"]
                products = [product["apiproduct"] for product in cred["apiProducts"]]
                resp = self.create_a_consumer_key_and_secret(
                    app["developerId"],
                    consumer_key=consumer_key,
                    consumer_secret=consumer_secret,
                    products=products,
                )
        return self.get_developer_app_details(app["developerId"])



#!/usr/bin/env python

# Copyright 2019 Matthew Delotavo
# Copyright 2015 Apigee Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import io
import base64
import getopt
import http.client
import json
import os
import re
import sys
import urllib.parse
import xml.dom.minidom
import zipfile

from apigee import APIGEE_ADMIN_API_URL

# from apigee.util import mfa_with_pyotp
from apigee.util import authorization, console

ApigeeHost = APIGEE_ADMIN_API_URL
UserPW = None
Directory = None
Organization = None
Environment = None
Name = None
BasePath = "/"
ShouldDeploy = True
ShouldOverride = False
GracePeriod = 15
Auth = None

url = urllib.parse.urlparse(ApigeeHost)
httpScheme = url[0]
httpHost = url[1]

body = None


class Struct:
    def __init__(self, **entries):
        self.__dict__.update(entries)


def httpCall(verb, uri, headers, body):
    if httpScheme == "https":
        conn = http.client.HTTPSConnection(httpHost)
    else:
        conn = http.client.HTTPConnection(httpHost)

    if headers == None:
        hdrs = {}
    else:
        hdrs = headers

    # if Auth:
    #     # hdrs['Authorization'] = 'Bearer %s' % AccessToken
    #     hdrs = authorization.set_header(hdrs, Auth)
    # else:
    #     hdrs['Authorization'] = 'Basic %s' % base64.b64encode(UserPW.encode()).decode()
    hdrs = authorization.set_header(hdrs, Auth)
    conn.request(verb, uri, body, hdrs)

    return conn.getresponse()


def getElementText(n):
    c = n.firstChild
    str = io.StringIO()

    while c != None:
        if c.nodeType == xml.dom.Node.TEXT_NODE:
            str.write(c.data)
        c = c.nextSibling

    return str.getvalue().strip()


def getElementVal(n, name):
    c = n.firstChild

    while c != None:
        if c.nodeName == name:
            return getElementText(c)
        c = c.nextSibling

    return None


# Return TRUE if any component of the file path contains a directory name that
# starts with a "." like '.svn', but not '.' or '..'
def pathContainsDot(p):
    c = re.compile("\.\w+")

    for pc in p.split("/"):
        if c.match(pc) != None:
            return True

    return False


def getDeployments():
    # Print info on deployments
    hdrs = {"Accept": "application/xml"}
    resp = httpCall(
        "GET",
        "/v1/organizations/%s/apis/%s/deployments" % (Organization, Name),
        hdrs,
        None,
    )

    if resp.status != 200:
        return None

    ret = []
    deployments = xml.dom.minidom.parse(resp)
    environments = deployments.getElementsByTagName("Environment")

    for env in environments:
        envName = env.getAttribute("name")
        revisions = env.getElementsByTagName("Revision")
        for rev in revisions:
            revNum = int(rev.getAttribute("name"))
            error = None
            state = getElementVal(rev, "State")
            basePaths = rev.getElementsByTagName("BasePath")

            if len(basePaths) > 0:
                basePath = getElementText(basePaths[0])
            else:
                basePath = "unknown"

            # svrs = rev.getElementsByTagName('Server')
            status = {
                "environment": envName,
                "revision": revNum,
                "basePath": basePath,
                "state": state,
            }

            if error != None:
                status["error"] = error

            ret.append(status)

    return ret


def printDeployments(dep, check_revision=None):
    if check_revision:
        revisions = [d["revision"] for d in dep]
        if check_revision not in revisions:
            sys.exit("Error: proxy version %i not found" % check_revision)
        console.log("Proxy version %i found" % check_revision)
    for d in dep:
        console.log("Environment: %s" % d["environment"])
        console.log("  Revision: %i BasePath = %s" % (d["revision"], d["basePath"]))
        console.log("  State: %s" % d["state"])
        if d["state"] != "deployed":
            sys.exit(1)
        if "error" in d:
            console.log("  Error: %s" % d["error"])


def deploy(args):
    global UserPW
    global Directory
    global Organization
    global Environment
    global Name
    global ShouldDeploy
    global ShouldOverride
    global Auth

    # ApigeeHost = 'https://api.enterprise.apigee.com'
    UserPW = args.username + ":" + args.password
    Directory = args.directory
    Organization = args.org
    Environment = args.environment
    Name = args.name
    # BasePath = '/'
    ShouldDeploy = not args.import_only
    ShouldOverride = args.seamless_deploy
    # GracePeriod = 15
    # AccessToken = mfa_with_pyotp.get_access_token(args)
    Auth = Struct(
        username=args.username, password=args.password, mfa_secret=args.mfa_secret
    )

    # if UserPW == None or \
    #         (Directory == None and ZipFile == None) or \
    #         Environment == None or \
    #         Name == None or \
    #         Organization == None:
    #     print """Usage: deploy -n [name] (-d [directory name] | -z [zipfile])
    #               -e [environment] -u [username:password] -o [organization]
    #               [-p [base path] -h [apigee API url] -i]
    #     base path defaults to "/"
    #     Apigee URL defaults to "https://api.enterprise.apigee.com"
    #     -i denotes to import only and not actually deploy
    #     """
    #     sys.exit(1)

    # url = urlparse.urlparse(ApigeeHost)
    # httpScheme = url[0]
    # httpHost = url[1]
    #
    # body = None

    if Directory != None:
        # Construct a ZIPped copy of the bundle in memory
        tf = io.BytesIO()
        zipout = zipfile.ZipFile(tf, "w")

        dirList = os.walk(Directory)
        for dirEntry in dirList:
            if not pathContainsDot(dirEntry[0]):
                for fileEntry in dirEntry[2]:
                    if not fileEntry.endswith("~"):
                        fn = os.path.join(dirEntry[0], fileEntry)
                        en = os.path.join(
                            os.path.relpath(dirEntry[0], Directory), fileEntry
                        )
                        console.log("Writing %s to %s" % (fn, en))
                        zipout.write(fn, en)

        zipout.close()
        body = tf.getvalue()
    elif ZipFile != None:
        f = open(ZipFile, "r")
        body = f.read()
        f.close()

    # Upload the bundle to the API
    hdrs = {"Content-Type": "application/octet-stream", "Accept": "application/json"}
    uri = "/v1/organizations/%s/apis?action=import&name=%s" % (Organization, Name)
    resp = httpCall("POST", uri, hdrs, body)

    if resp.status != 200 and resp.status != 201:
        console.log(
            "Import failed to %s with status %i:\n%s"
            % (uri, resp.status, resp.read().decode())
        )
        sys.exit(2)

    deployment = json.loads(resp.read().decode())
    revision = int(deployment["revision"])

    console.log("Imported new proxy version %i" % revision)

    if ShouldDeploy and not ShouldOverride:
        # Undeploy duplicates
        deps = getDeployments()
        for d in deps:
            if (
                d["environment"] == Environment
                and d["basePath"] == BasePath
                and d["revision"] != revision
            ):
                console.log(
                    "Undeploying revision %i in same environment and path:"
                    % d["revision"]
                )
                conn = http.client.HTTPSConnection(httpHost)
                resp = httpCall(
                    "POST",
                    (
                        "/v1/organizations/%s/apis/%s/deployments"
                        + "?action=undeploy"
                        + "&env=%s"
                        + "&revision=%i"
                    )
                    % (Organization, Name, Environment, d["revision"]),
                    None,
                    None,
                )
                if resp.status != 200 and resp.status != 204:
                    console.log(
                        "Error %i on undeployment:\n%s"
                        % (resp.status, resp.read().decode())
                    )

        # Deploy the bundle
        hdrs = {"Accept": "application/json"}
        resp = httpCall(
            "POST",
            (
                "/v1/organizations/%s/apis/%s/deployments"
                + "?action=deploy"
                + "&env=%s"
                + "&revision=%i"
                + "&basepath=%s"
            )
            % (Organization, Name, Environment, revision, BasePath),
            hdrs,
            None,
        )

        if resp.status != 200 and resp.status != 201:
            console.log(
                "Deploy failed with status %i:\n%s"
                % (resp.status, resp.read().decode())
            )
            sys.exit(2)

    if ShouldOverride:
        # Seamless Deploy the bundle
        console.log("Seamless deploy %s" % Name)
        hdrs = {"Content-Type": "application/x-www-form-urlencoded"}
        resp = httpCall(
            "POST",
            (
                "/v1/organizations/%s/environments/%s/apis/%s/revisions/%s/deployments"
                + "?override=true"
                + "&delay=%s"
            )
            % (Organization, Environment, Name, revision, GracePeriod),
            hdrs,
            None,
        )

        if resp.status != 200 and resp.status != 201:
            console.log(
                "Deploy failed with status %i:\n%s"
                % (resp.status, resp.read().decode())
            )
            sys.exit(2)

    deps = getDeployments()
    # printDeployments(deps, check_revision=revision)
    if ShouldDeploy and not ShouldOverride:
        printDeployments(deps)
    if ShouldOverride:
        printDeployments(deps, check_revision=revision)


def main():
    pass


if __name__ == "__main__":
    main()



import argparse

from apigee.api.deployments import Deployments

from apigee.parsers.parent_parser import ParentParser
from apigee.parsers.silent_parser import SilentParser
from apigee.parsers.verbose_parser import VerboseParser

from apigee.util import console


class ParserDeployments:
    def __init__(self, parser, **kwargs):
        self._parser = parser
        self._parser_deployments = self._parser.add_parser(
            "deployments", aliases=["deps"], help="see API deployments"
        ).add_subparsers()
        self._parent_parser = kwargs.get("parent_parser", ParentParser())
        self._silent_parser = kwargs.get("silent_parser", SilentParser())
        self._verbose_parser = kwargs.get("verbose_parser", VerboseParser())
        self._create_parser()

    @property
    def parser(self):
        return self._parser

    @parser.setter
    def parser(self, value):
        self._parser = value

    @property
    def parser_deployments(self):
        return self._parser_deployments

    @parser_deployments.setter
    def parser_deployments(self, value):
        self._parser_deployments = value

    @property
    def parent_parser(self):
        return self._parent_parser

    @parent_parser.setter
    def parent_parser(self, value):
        self._parent_parser = value

    def __call__(self):
        return self._parser

    def _build_get_api_proxy_deployment_details_argument(self):
        get_api_proxy_deployment_details = self._parser_deployments.add_parser(
            "get",
            aliases=["get-api-proxy-deployment-details"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Returns detail on all deployments of the API proxy for all environments. All deployments are listed in the test and prod environments, as well as other environments, if they exist.",
        )
        get_api_proxy_deployment_details.add_argument(
            "-n", "--name", help="name", required=True
        )
        get_api_proxy_deployment_details.add_argument(
            "-r", "--revision-name", action="store_true", help="get revisions only"
        )
        get_api_proxy_deployment_details.add_argument(
            "-j",
            "--json",
            action="store_const",
            const="json",
            help="use json output",
            default="table",
        )
        # get_api_proxy_deployment_details.add_argument('--max-colwidth', help='max column width', type=int, default=40)
        get_api_proxy_deployment_details.add_argument(
            "--showindex", action="store_true", help="add row indices"
        )
        get_api_proxy_deployment_details.add_argument(
            "--tablefmt",
            help="defines how the table is formatted",
            default="plain",
            choices=(
                "plain",
                "simple",
                "github",
                "grid",
                "fancy_grid",
                "pipe",
                "orgtbl",
                "jira",
                "presto",
                "psql",
                "rst",
                "mediawiki",
                "moinmoin",
                "youtrack",
                "html",
                "latex",
                "latex_raw",
                "latex_booktabs",
                "textile",
            ),
            type=str,
        )
        get_api_proxy_deployment_details.set_defaults(
            func=lambda args: console.log(
                Deployments(args, args.org, args.name).get_api_proxy_deployment_details(
                    formatted=True,
                    format=args.json,
                    showindex=args.showindex,
                    tablefmt=args.tablefmt,
                    revision_name_only=args.revision_name,
                )
            )
        )

    def _create_parser(self):
        self._build_get_api_proxy_deployment_details_argument()



from setuptools import setup, find_packages

from apigee import APP, __version__, description

# read the contents of your README file
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.rst"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name=APP,
    version=__version__,
    description=description,
    long_description=long_description,
    long_description_content_type="text/x-rst",
    author="Matthew Delotavo",
    author_email="matthew.t.delotavo@gmail.com",
    url="https://github.com/mdelotavo/apigee-cli",
    download_url=f"https://github.com/mdelotavo/apigee-cli/archive/v{__version__}.tar.gz",
    keywords=["apigee", "management", "api", "oauth", "mfa"],
    packages=find_packages(),
    entry_points={"console_scripts": ["apigee=apigee.__main__:main"]},
    python_requires=">=3.6",
    install_requires=["pyotp", "requests", "tqdm", "tabulate", "pyjwt"],
    license="Apache license 2.0",
    classifiers=[
        "Development Status :: 4 - Beta",  # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        "Intended Audience :: Developers",
        "Topic :: Utilities",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    project_urls={"Documentation": "https://mdelotavo.github.io/apigee-cli/index.html"},
)



import argparse

from apigee.api.developers import Developers

from apigee.parsers.parent_parser import ParentParser
from apigee.parsers.prefix_parser import PrefixParser
from apigee.parsers.silent_parser import SilentParser
from apigee.parsers.verbose_parser import VerboseParser

from apigee.util import console


class ParserDevelopers:
    def __init__(self, parser, **kwargs):
        self._parser = parser
        self._parser_developers = self._parser.add_parser(
            "developers", aliases=["devs"], help="manage developers"
        ).add_subparsers()
        self._parent_parser = kwargs.get("parent_parser", ParentParser())
        self._prefix_parser = kwargs.get(
            "prefix_parser", PrefixParser(profile="default")
        )
        self._silent_parser = kwargs.get("silent_parser", SilentParser())
        self._verbose_parser = kwargs.get("verbose_parser", VerboseParser())
        self._create_parser()

    @property
    def parser(self):
        return self._parser

    @parser.setter
    def parser(self, value):
        self._parser = value

    @property
    def parser_developers(self):
        return self._parser_developers

    @parser_developers.setter
    def parser_developers(self, value):
        self._parser_developers = value

    @property
    def parent_parser(self):
        return self._parent_parser

    @parent_parser.setter
    def parent_parser(self, value):
        self._parent_parser = value

    @property
    def prefix_parser(self):
        return self._prefix_parser

    @prefix_parser.setter
    def prefix_parser(self, value):
        self._prefix_parser = value

    def __call__(self):
        return self._parser

    def _build_create_developer_argument(self):
        create_developer = self._parser_developers.add_parser(
            "create",
            aliases=["create-developer"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._prefix_parser(),
            ],
            help="Creates a profile for a developer in an organization. Once created, the developer can register an app and receive an API key.",
        )
        create_developer.add_argument(
            "-n",
            "--name",
            help="The developer's email. This value is used to uniquely identify the developer in Apigee Edge.",
            required=True,
        )
        create_developer.add_argument(
            "--first-name", help="The first name of the developer.", required=True
        )
        create_developer.add_argument(
            "--last-name", help="The last name of the developer.", required=True
        )
        create_developer.add_argument(
            "--user-name",
            help="The developer's username. This value is not used by Apigee Edge.",
            required=True,
        )
        create_developer.add_argument(
            "--attributes",
            help="request body e.g.: '{\"attributes\" : [ ]}'",
            required=False,
            default='{"attributes" : [ ]}',
        )
        create_developer.set_defaults(
            func=lambda args: console.log(
                Developers(args, args.org, args.name)
                .create_developer(
                    args.first_name,
                    args.last_name,
                    args.user_name,
                    attributes=args.attributes,
                )
                .text
            )
        )

    def _build_delete_developer_argument(self):
        delete_developer = self._parser_developers.add_parser(
            "delete",
            aliases=["delete-developer"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._prefix_parser(),
            ],
            help="Deletes a developer from an organization. All apps and API keys associated with the developer are also removed from the organization. All times in the response are UNIX times. To avoid permanently deleting developers and their artifacts, consider deactivating developers instead.",
        )
        delete_developer.add_argument(
            "-n",
            "--name",
            help="The developer's email. This value is used to uniquely identify the developer in Apigee Edge.",
            required=True,
        )
        delete_developer.set_defaults(
            func=lambda args: console.log(
                Developers(args, args.org, args.name).delete_developer().text
            )
        )

    def _build_get_developer_argument(self):
        get_developer = self._parser_developers.add_parser(
            "get",
            aliases=["get-developer"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._prefix_parser(),
            ],
            help="Returns the profile for a developer by email address or ID. All time values are UNIX time values. The profile includes the developer's email address, ID, name, and other information.",
        )
        get_developer.add_argument(
            "-n",
            "--name",
            help="The developer's email. This value is used to uniquely identify the developer in Apigee Edge.",
            required=True,
        )
        get_developer.set_defaults(
            func=lambda args: console.log(
                Developers(args, args.org, args.name).get_developer().text
            )
        )

    def _build_get_developer_by_app_argument(self):
        get_developer_by_app = self._parser_developers.add_parser(
            "get-by-app",
            aliases=["get-developer-by-app"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._prefix_parser(),
            ],
            help="Gets the developer profile by app name. The profile retrieved is for the developer associated with the app in the organization. All time values are UNIX time values.",
        )
        get_developer_by_app.add_argument(
            "--app-name", help="the app name", required=True
        )
        get_developer_by_app.set_defaults(
            func=lambda args: console.log(
                Developers(args, args.org, None)
                .get_developer_by_app(args.app_name)
                .text
            )
        )

    def _build_list_developers_argument(self):
        list_developers = self._parser_developers.add_parser(
            "list",
            aliases=["list-developers"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._prefix_parser(),
            ],
            help="Lists all developers in an organization by email address. This call does not list any company developers who are a part of the designated organization.",
        )
        list_developers.add_argument(
            "--expand",
            action="store_true",
            help="Set to true to list developers exanded with details.",
        )
        list_developers.add_argument(
            "--count",
            default=1000,
            type=int,
            help="Limits the list to the number you specify. Use with the startKey parameter to provide more targeted filtering. The limit is 1000.",
        )
        list_developers.add_argument(
            "--startkey",
            default="",
            help="To filter the keys that are returned, enter the email of a developer that the list will start with.",
        )
        list_developers.set_defaults(
            func=lambda args: console.log(
                Developers(args, args.org, None).list_developers(
                    prefix=args.prefix,
                    expand=args.expand,
                    count=args.count,
                    startkey=args.startkey,
                )
            )
        )

    def _build_set_developer_status_argument(self):
        set_developer_status = self._parser_developers.add_parser(
            "set-status",
            aliases=["set-developer-status"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._prefix_parser(),
            ],
            help="Sets a developer's status to active or inactive for a specific organization. Run this API for each organization where you want to change the developer's status.",
        )
        set_developer_status.add_argument(
            "-n",
            "--name",
            help="The developer's email. This value is used to uniquely identify the developer in Apigee Edge.",
            required=True,
        )
        set_developer_status.add_argument(
            "--action", choices=("active", "inactive"), type=str, help="", required=True
        )
        set_developer_status.set_defaults(
            func=lambda args: console.log(
                Developers(args, args.org, args.name)
                .set_developer_status(args.action)
                .text
            )
        )

    def _build_update_developer_argument(self):
        set_developer_status = self._parser_developers.add_parser(
            "update",
            aliases=["update-developer"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._prefix_parser(),
            ],
            help="Update an existing developer profile. To add new values or update existing values, submit the new or updated portion of the developer profile along with the rest of the developer profile, even if no values are changing. To delete attributes from a developer profile, submit the entire profile without the attributes that you want to delete.",
        )
        set_developer_status.add_argument(
            "-n",
            "--name",
            help="The developer's email. This value is used to uniquely identify the developer in Apigee Edge.",
            required=True,
        )
        set_developer_status.add_argument(
            "-b", "--body", help="request body", required=True
        )
        set_developer_status.set_defaults(
            func=lambda args: console.log(
                Developers(args, args.org, args.name).update_developer(args.body).text
            )
        )

    def _build_update_a_developer_attribute_argument(self):
        update_a_developer_attribute = self._parser_developers.add_parser(
            "update-attr",
            aliases=["update-attribute", "update-a-developer-attribute"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Updates the value of a developer attribute.",
        )
        update_a_developer_attribute.add_argument(
            "-n", "--name", help="the developer's email address", required=True
        )
        update_a_developer_attribute.add_argument(
            "--attribute-name", help="attribute name", required=True
        )
        update_a_developer_attribute.add_argument(
            "--updated-value", help="updated value", required=True
        )
        update_a_developer_attribute.set_defaults(
            func=lambda args: console.log(
                Developers(args, args.org, args.name)
                .update_a_developer_attribute(args.attribute_name, args.updated_value)
                .text
            )
        )

    def _build_delete_developer_attribute_argument(self):
        delete_developer_attribute = self._parser_developers.add_parser(
            "delete-attr",
            aliases=["delete-attribute", "delete-developer-attribute"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Deletes a developer attribute.",
        )
        delete_developer_attribute.add_argument(
            "-n", "--name", help="the developer's email address", required=True
        )
        delete_developer_attribute.add_argument(
            "--attribute-name", help="attribute name", required=True
        )
        delete_developer_attribute.set_defaults(
            func=lambda args: console.log(
                Developers(args, args.org, args.name)
                .delete_developer_attribute(args.attribute_name)
                .text
            )
        )

    def _build_get_all_developer_attributes_argument(self):
        get_all_developer_attributes = self._parser_developers.add_parser(
            "get-attrs",
            aliases=["get-attributes", "get-all-developer-attributes"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Returns a list of all developer attributes.",
        )
        get_all_developer_attributes.add_argument(
            "-n", "--name", help="the developer's email address", required=True
        )
        get_all_developer_attributes.set_defaults(
            func=lambda args: console.log(
                Developers(args, args.org, args.name)
                .get_all_developer_attributes()
                .text
            )
        )

    def _build_update_all_developer_attributes_argument(self):
        update_all_developer_attributes = self._parser_developers.add_parser(
            "update-all-attrs",
            aliases=["update-all-attributes", "update-all-developer-attributes"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Updates or creates developer attributes. This API replaces the current list of attributes with the attributes specified in the request body. This lets you update existing attributes, add new attributes, or delete existing attributes by omitting them from the request body.",
        )
        update_all_developer_attributes.add_argument(
            "-n", "--name", help="the developer's email address", required=True
        )
        update_all_developer_attributes.add_argument(
            "-b", "--body", help="request body", required=True
        )
        update_all_developer_attributes.set_defaults(
            func=lambda args: console.log(
                Developers(args, args.org, args.name)
                .update_all_developer_attributes(args.body)
                .text
            )
        )

    def _create_parser(self):
        self._build_create_developer_argument()
        self._build_delete_developer_argument()
        self._build_get_developer_argument()
        self._build_get_developer_by_app_argument()
        self._build_list_developers_argument()
        self._build_set_developer_status_argument()
        self._build_update_developer_argument()
        self._build_update_a_developer_attribute_argument()
        self._build_delete_developer_attribute_argument()
        self._build_get_all_developer_attributes_argument()
        self._build_update_all_developer_attributes_argument()



#!/usr/bin/env python
"""https://apidocs.apigee.com/api-services/content/environment-keyvalue-maps"""

import json
from abc import ABC, abstractmethod


class IKeyvaluemaps:
    def __init__(self, auth, org_name, map_name):
        self._auth = auth
        self._org_name = org_name
        self._map_name = map_name

    def __call__(self):
        pass

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @property
    def map_name(self):
        return self._map_name

    @map_name.setter
    def map_name(self, value):
        self._map_name = value

    @abstractmethod
    def create_keyvaluemap_in_an_environment(self, environment, request_body):
        pass

    @abstractmethod
    def delete_keyvaluemap_from_an_environment(self, environment):
        pass

    @abstractmethod
    def delete_keyvaluemap_entry_in_an_environment(self, environment, entry_name):
        pass

    @abstractmethod
    def get_keyvaluemap_in_an_environment(self, environment):
        pass

    @abstractmethod
    def get_a_keys_value_in_an_environment_scoped_keyvaluemap(
        self, environment, entry_name
    ):
        pass

    @abstractmethod
    def list_keyvaluemaps_in_an_environment(self, environment, prefix=None):
        pass

    @abstractmethod
    def update_keyvaluemap_in_an_environment(self, environment, request_body):
        pass

    @abstractmethod
    def create_an_entry_in_an_environment_scoped_kvm(
        self, environment, entry_name, entry_value
    ):
        pass

    @abstractmethod
    def update_an_entry_in_an_environment_scoped_kvm(
        self, environment, entry_name, updated_value
    ):
        pass

    @abstractmethod
    def list_keys_in_an_environment_scoped_keyvaluemap(
        self, environment, startkey, count
    ):
        pass

    @abstractmethod
    def push_keyvaluemap(self, environment, file):
        pass


class KeyvaluemapsSerializer:
    def serialize_details(self, maps, format, prefix=None):
        resp = maps
        if format == "text":
            return maps.text
        maps = maps.json()
        if prefix:
            maps = [map for map in maps if map.startswith(prefix)]
        if format == "json":
            return json.dumps(maps)
        elif format == "table":
            pass
        # else:
        #     raise ValueError(format)
        return resp



import argparse

from apigee.api.keyvaluemaps import Keyvaluemaps

from apigee.parsers.parent_parser import ParentParser
from apigee.parsers.file_parser import FileParser
from apigee.parsers.environment_parser import EnvironmentParser
from apigee.parsers.prefix_parser import PrefixParser
from apigee.parsers.silent_parser import SilentParser
from apigee.parsers.verbose_parser import VerboseParser

from apigee.util import console


class ParserKeyvaluemaps:
    def __init__(self, parser, **kwargs):
        self._parser = parser
        self._parser_keyvaluemaps = self._parser.add_parser(
            "keyvaluemaps", aliases=["kvms"], help="manage keyvaluemaps"
        ).add_subparsers()
        self._parent_parser = kwargs.get("parent_parser", ParentParser())
        self._file_parser = kwargs.get("file_parser", FileParser())
        self._environment_parser = kwargs.get("environment_parser", EnvironmentParser())
        self._prefix_parser = kwargs.get(
            "prefix_parser", PrefixParser(profile="default")
        )
        self._silent_parser = kwargs.get("silent_parser", SilentParser())
        self._verbose_parser = kwargs.get("verbose_parser", VerboseParser())
        self._create_parser()

    @property
    def parser(self):
        return self._parser

    @parser.setter
    def parser(self, value):
        self._parser = value

    @property
    def parser_keyvaluemaps(self):
        return self._parser_keyvaluemaps

    @parser_keyvaluemaps.setter
    def parser_keyvaluemaps(self, value):
        self._parser_keyvaluemaps = value

    @property
    def parent_parser(self):
        return self._parent_parser

    @parent_parser.setter
    def parent_parser(self, value):
        self._parent_parser = value

    @property
    def file_parser(self):
        return self._file_parser

    @file_parser.setter
    def file_parser(self, value):
        self._file_parser = value

    @property
    def environment_parser(self):
        return self._environment_parser

    @environment_parser.setter
    def environment_parser(self, value):
        self._environment_parser = value

    @property
    def prefix_parser(self):
        return self._prefix_parser

    @prefix_parser.setter
    def prefix_parser(self, value):
        self._prefix_parser = value

    def __call__(self):
        return self._parser

    def _build_create_keyvaluemap_in_an_environment_argument(self):
        create_keyvaluemap_in_an_environment = self._parser_keyvaluemaps.add_parser(
            "create",
            aliases=["create-keyvaluemap-in-an-environment"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Creates a key value map in an environment.",
        )
        create_keyvaluemap_in_an_environment.add_argument(
            "-b", "--body", help="request body", required=True
        )
        create_keyvaluemap_in_an_environment.set_defaults(
            func=lambda args: console.log(
                Keyvaluemaps(args, args.org, args.name)
                .create_keyvaluemap_in_an_environment(args.environment, args.body)
                .text
            )
        )

    def _build_delete_keyvaluemap_from_an_environment_argument(self):
        delete_keyvaluemap_from_an_environment = self._parser_keyvaluemaps.add_parser(
            "delete",
            aliases=["delete-keyvaluemap-from-an-environment"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Deletes a key/value map and all associated entries from an environment.",
        )
        delete_keyvaluemap_from_an_environment.add_argument(
            "-n", "--name", help="name", required=True
        )
        delete_keyvaluemap_from_an_environment.set_defaults(
            func=lambda args: console.log(
                Keyvaluemaps(args, args.org, args.name)
                .delete_keyvaluemap_from_an_environment(args.environment)
                .text
            )
        )

    def _build_delete_keyvaluemap_entry_in_an_environment_argument(self):
        delete_keyvaluemap_entry_in_an_environment = self._parser_keyvaluemaps.add_parser(
            "delete-entry",
            aliases=["delete-keyvaluemap-entry-in-an-environment"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Deletes a specific key/value map entry in an environment by name, along with associated entries.",
        )
        delete_keyvaluemap_entry_in_an_environment.add_argument(
            "-n", "--name", help="name", required=True
        )
        delete_keyvaluemap_entry_in_an_environment.add_argument(
            "--entry-name", help="entry name", required=True
        )
        delete_keyvaluemap_entry_in_an_environment.set_defaults(
            func=lambda args: console.log(
                Keyvaluemaps(args, args.org, args.name)
                .delete_keyvaluemap_entry_in_an_environment(
                    args.environment, args.entry_name
                )
                .text
            )
        )

    def _build_get_keyvaluemap_in_an_environment_argument(self):
        get_keyvaluemap_in_an_environment = self._parser_keyvaluemaps.add_parser(
            "get",
            aliases=["get-keyvaluemap-in-an-environment"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Gets a KeyValueMap (KVM) in an environment by name, along with the keys and values.",
        )
        get_keyvaluemap_in_an_environment.add_argument(
            "-n", "--name", help="name", required=True
        )
        get_keyvaluemap_in_an_environment.set_defaults(
            func=lambda args: console.log(
                Keyvaluemaps(args, args.org, args.name)
                .get_keyvaluemap_in_an_environment(args.environment)
                .text
            )
        )

    def _build_get_a_keys_value_in_an_environment_scoped_keyvaluemap_argument(self):
        get_a_keys_value_in_an_environment_scoped_keyvaluemap = self._parser_keyvaluemaps.add_parser(
            "get-value",
            aliases=["get-a-keys-value-in-an-environment-scoped-keyvaluemap"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Gets the value of a key in an environment-scoped KeyValueMap (KVM).",
        )
        get_a_keys_value_in_an_environment_scoped_keyvaluemap.add_argument(
            "-n", "--name", help="name", required=True
        )
        get_a_keys_value_in_an_environment_scoped_keyvaluemap.add_argument(
            "--entry-name", help="entry name", required=True
        )
        get_a_keys_value_in_an_environment_scoped_keyvaluemap.set_defaults(
            func=lambda args: console.log(
                Keyvaluemaps(args, args.org, args.name)
                .get_a_keys_value_in_an_environment_scoped_keyvaluemap(
                    args.environment, args.entry_name
                )
                .text
            )
        )

    def _build_list_keyvaluemaps_in_an_environment_argument(self):
        list_keyvaluemaps_in_an_environment = self._parser_keyvaluemaps.add_parser(
            "list",
            aliases=["list-keyvaluemaps-in-an-environment"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
                self._prefix_parser(),
            ],
            help="Lists the name of all key/value maps in an environment and optionally returns an expanded view of all key/value maps for the environment.",
        )
        list_keyvaluemaps_in_an_environment.set_defaults(
            func=lambda args: console.log(
                Keyvaluemaps(args, args.org, None).list_keyvaluemaps_in_an_environment(
                    args.environment, prefix=args.prefix
                )
            )
        )

    def _build_update_keyvaluemap_in_an_environment_argument(self):
        update_keyvaluemap_in_an_environment = self._parser_keyvaluemaps.add_parser(
            "update",
            aliases=["update-keyvaluemap-in-an-environment"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Note: This API is supported for Apigee Edge for Private Cloud only. For Apigee Edge for Public Cloud use Update an entry in an environment-scoped KVM. Updates an existing KeyValueMap in an environment. Does not override the existing map. Instead, this method updates the entries if they exist or adds them if not. It can take several minutes before the new value is visible to runtime traffic.",
        )
        update_keyvaluemap_in_an_environment.add_argument(
            "-n", "--name", help="name", required=True
        )
        update_keyvaluemap_in_an_environment.add_argument(
            "-b", "--body", help="request body", required=True
        )
        update_keyvaluemap_in_an_environment.set_defaults(
            func=lambda args: console.log(
                Keyvaluemaps(args, args.org, args.name)
                .update_keyvaluemap_in_an_environment(args.environment, args.body)
                .text
            )
        )

    def _build_create_an_entry_in_an_environment_scoped_kvm_argument(self):
        create_an_entry_in_an_environment_scoped_kvm = self._parser_keyvaluemaps.add_parser(
            "create-entry",
            aliases=["create-an-entry-in-an-environment-scoped-kvm"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Note: This API is supported for Apigee Edge for the Public Cloud only. Creates an entry in an existing KeyValueMap scoped to an environment. A key (name) cannot be larger than 2 KB. KVM names are case sensitive.",
        )
        create_an_entry_in_an_environment_scoped_kvm.add_argument(
            "-n", "--name", help="name", required=True
        )
        create_an_entry_in_an_environment_scoped_kvm.add_argument(
            "--entry-name", help="entry name", required=True
        )
        create_an_entry_in_an_environment_scoped_kvm.add_argument(
            "--entry-value", help="entry value", required=True
        )
        create_an_entry_in_an_environment_scoped_kvm.set_defaults(
            func=lambda args: console.log(
                Keyvaluemaps(args, args.org, args.name)
                .create_an_entry_in_an_environment_scoped_kvm(
                    args.environment, args.entry_name, args.entry_value
                )
                .text
            )
        )

    def _build_update_an_entry_in_an_environment_scoped_kvm_argument(self):
        update_an_entry_in_an_environment_scoped_kvm = self._parser_keyvaluemaps.add_parser(
            "update-entry",
            aliases=["update-an-entry-in-an-environment-scoped-kvm"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Note: This API is supported for Apigee Edge for the Public Cloud only. Updates an entry in a KeyValueMap scoped to an environment. A key cannot be larger than 2 KB. KVM names are case sensitive. Does not override the existing map. It can take several minutes before the new value is visible to runtime traffic.",
        )
        update_an_entry_in_an_environment_scoped_kvm.add_argument(
            "-n", "--name", help="name", required=True
        )
        update_an_entry_in_an_environment_scoped_kvm.add_argument(
            "--entry-name", help="entry name", required=True
        )
        update_an_entry_in_an_environment_scoped_kvm.add_argument(
            "--updated-value", help="updated value", required=True
        )
        update_an_entry_in_an_environment_scoped_kvm.set_defaults(
            func=lambda args: console.log(
                Keyvaluemaps(args, args.org, args.name)
                .update_an_entry_in_an_environment_scoped_kvm(
                    args.environment, args.entry_name, args.updated_value
                )
                .text
            )
        )

    def _build_list_keys_in_an_environment_scoped_keyvaluemap_argument(self):
        list_keys_in_an_environment_scoped_keyvaluemap = self._parser_keyvaluemaps.add_parser(
            "list-keys",
            aliases=["list-keys-in-an-environment-scoped-keyvaluemap"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
                self._prefix_parser(),
            ],
            help="Note: This API is supported for Apigee Edge for the Public Cloud only. Lists keys in a KeyValueMap scoped to an environment. KVM names are case sensitive.",
        )
        list_keys_in_an_environment_scoped_keyvaluemap.add_argument(
            "-n", "--name", help="name", required=True
        )
        list_keys_in_an_environment_scoped_keyvaluemap.add_argument(
            "--startkey",
            default="",
            help="To filter the keys that are returned, enter the name of a key that the list will start with.",
        )
        list_keys_in_an_environment_scoped_keyvaluemap.add_argument(
            "--count",
            default=100,
            type=int,
            help="Limits the list of keys to the number you specify, up to a maximum of 100. Use with the startkey parameter to provide more targeted filtering.",
        )
        list_keys_in_an_environment_scoped_keyvaluemap.set_defaults(
            func=lambda args: console.log(
                Keyvaluemaps(args, args.org, args.name)
                .list_keys_in_an_environment_scoped_keyvaluemap(
                    args.environment, args.startkey, args.count
                )
                .text
            )
        )

    def _build_push_keyvaluemap_argument(self):
        push_keyvaluemap = self._parser_keyvaluemaps.add_parser(
            "push",
            aliases=["push-keyvaluemap"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
                self._file_parser(),
            ],
            help="Push KeyValueMap to Apigee. This will create KeyValueMap/entries if they do not exist, update existing KeyValueMap/entries, and delete entries on Apigee that are not present in the request body.",
        )
        # push_keyvaluemap.add_argument('-n', '--name', help='name', required=True)
        push_keyvaluemap.set_defaults(
            func=lambda args: Keyvaluemaps(args, args.org, None).push_keyvaluemap(
                args.environment, args.file
            )
        )

    def _create_parser(self):
        self._build_create_keyvaluemap_in_an_environment_argument()
        self._build_delete_keyvaluemap_from_an_environment_argument()
        self._build_delete_keyvaluemap_entry_in_an_environment_argument()
        self._build_get_keyvaluemap_in_an_environment_argument()
        self._build_get_a_keys_value_in_an_environment_scoped_keyvaluemap_argument()
        self._build_list_keyvaluemaps_in_an_environment_argument()
        self._build_update_keyvaluemap_in_an_environment_argument()
        self._build_create_an_entry_in_an_environment_scoped_kvm_argument()
        self._build_update_an_entry_in_an_environment_scoped_kvm_argument()
        self._build_list_keys_in_an_environment_scoped_keyvaluemap_argument()
        self._build_push_keyvaluemap_argument()



#!/usr/bin/env python
"""https://apidocs.apigee.com/api/stats"""

import json
import requests
import urllib.parse

from tabulate import tabulate

from apigee import APIGEE_ADMIN_API_URL
from apigee.abstract.api.stats import IStats, StatsSerializer
from apigee.util import authorization, console


class Stats(IStats):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def get_license_utilization(
        self,
        time_range,
        environments=["test", "prod"],
        tzo=None,
        api_calls_per_year=10000000000,
    ):
        metrics = {}
        for env in environments:
            uri = f"{APIGEE_ADMIN_API_URL}/v1/o/{self._org_name}/environments/{env}/stats?select=sum(message_count)&timeRange={urllib.parse.quote(time_range)}"
            params = {}
            if tzo:
                params["tzo"] = tzo
            hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
            resp = requests.get(uri, headers=hdrs, params=params)
            resp.raise_for_status()
            for detail in resp.json()["environments"]:
                if detail["name"] == env:
                    for metric in detail["metrics"]:
                        if metric["name"] == "sum(message_count)":
                            metrics[env] = int(float(metric["values"][0]))
        console.log()
        console.log(f"Apigee license per year: {api_calls_per_year:,} API calls")
        # console.log(
        #     f'Current license utilization per environment in "{self._org_name}" org: '
        # )
        console.log()
        table = [[k, f"{v:,}"] for k, v in metrics.items()]
        headers = ["environment", "API calls"]
        t = tabulate(table, headers)
        # t = tabulate(table, headers, showindex=showindex, tablefmt=tablefmt)
        console.log(t)
        console.log()
        total = sum(metrics.values())
        console.log(
            f"Total API calls for specified time range ({time_range}): {total:,}"
        )
        utilization = total * 100 / api_calls_per_year
        console.log(
            f"Utilization % for specified time range ({time_range}): {utilization}%"
        )
        console.log()
        return t



#!/usr/bin/env python
"""https://apidocs.apigee.com/api/api-products-1"""

import json
from abc import ABC, abstractmethod


class IApiproducts:
    def __init__(self, auth, org_name, apiproduct_name):
        self._auth = auth
        self._org_name = org_name
        self._apiproduct_name = apiproduct_name

    def __call__(self):
        pass

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @property
    def apiproduct_name(self):
        return self._apiproduct_name

    @apiproduct_name.setter
    def apiproduct_name(self, value):
        self._apiproduct_name = value

    @abstractmethod
    def create_api_product(self, request_body):
        pass

    @abstractmethod
    def delete_api_product(self):
        pass

    @abstractmethod
    def get_api_product(self):
        pass

    @abstractmethod
    def list_api_products(self, prefix=None, expand=False, count=1000, startkey=""):
        pass

    @abstractmethod
    def update_api_product(self, request_body):
        pass

    @abstractmethod
    def push_apiproducts(self, file):
        pass


class ApiproductsSerializer:
    def serialize_details(self, apiproducts, format, prefix=None):
        resp = apiproducts
        if format == "text":
            return apiproducts.text
        apiproducts = apiproducts.json()
        if prefix:
            apiproducts = [
                apiproduct
                for apiproduct in apiproducts
                if apiproduct.startswith(prefix)
            ]
        if format == "json":
            return json.dumps(apiproducts)
        elif format == "table":
            pass
        # else:
        #     raise ValueError(format)
        return resp



import argparse

from apigee.api.caches import Caches

from apigee.parsers.parent_parser import ParentParser
from apigee.parsers.file_parser import FileParser
from apigee.parsers.environment_parser import EnvironmentParser
from apigee.parsers.prefix_parser import PrefixParser
from apigee.parsers.silent_parser import SilentParser
from apigee.parsers.verbose_parser import VerboseParser

from apigee.util import console


class ParserCaches:
    def __init__(self, parser, **kwargs):
        self._parser = parser
        self._parser_caches = self._parser.add_parser(
            "caches", help="manage caches"
        ).add_subparsers()
        self._parent_parser = kwargs.get("parent_parser", ParentParser())
        self._file_parser = kwargs.get("file_parser", FileParser())
        self._environment_parser = kwargs.get("environment_parser", EnvironmentParser())
        self._prefix_parser = kwargs.get(
            "prefix_parser", PrefixParser(profile="default")
        )
        self._silent_parser = kwargs.get("silent_parser", SilentParser())
        self._verbose_parser = kwargs.get("verbose_parser", VerboseParser())
        self._create_parser()

    @property
    def parser(self):
        return self._parser

    @parser.setter
    def parser(self, value):
        self._parser = value

    @property
    def parser_caches(self):
        return self._parser_caches

    @parser_caches.setter
    def parser_caches(self, value):
        self._parser_caches = value

    @property
    def parent_parser(self):
        return self._parent_parser

    @parent_parser.setter
    def parent_parser(self, value):
        self._parent_parser = value

    @property
    def file_parser(self):
        return self._file_parser

    @file_parser.setter
    def file_parser(self, value):
        self._file_parser = value

    @property
    def environment_parser(self):
        return self._environment_parser

    @environment_parser.setter
    def environment_parser(self, value):
        self._environment_parser = value

    @property
    def prefix_parser(self):
        return self._prefix_parser

    @prefix_parser.setter
    def prefix_parser(self, value):
        self._prefix_parser = value

    def __call__(self):
        return self._parser

    def _build_clear_all_cache_entries_argument(self):
        parser = self._parser_caches.add_parser(
            "clear",
            aliases=["clear-all-cache-entries"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Clears all cache entries.",
        )
        parser.add_argument("-n", "--name", help="name", required=True)
        parser.set_defaults(
            func=lambda args: console.log(
                Caches(args, args.org, args.name)
                .clear_all_cache_entries(args.environment)
                .text
            )
        )

    def _build_clear_a_cache_entry_argument(self):
        parser = self._parser_caches.add_parser(
            "clear-entry",
            aliases=["clear-a-cache-entry"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Clears a cache entry, which is identified by the full CacheKey prefix and value.",
        )
        parser.add_argument("-n", "--name", help="name", required=True)
        parser.add_argument("--entry", help="cache entry to clear", required=True)
        parser.set_defaults(
            func=lambda args: console.log(
                Caches(args, args.org, args.name)
                .clear_a_cache_entry(args.environment, args.entry)
                .text
            )
        )

    def _build_create_a_cache_in_an_environment_argument(self):
        parser = self._parser_caches.add_parser(
            "create",
            aliases=["create-a-cache-in-an-environment"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Creates a cache in an environment. Caches are created per environment. For data segregation, a cache created in 'test', for example, cannot be accessed by API proxies deployed in 'prod'. The JSON object in the request body can be empty to create a cache with the default settings.",
        )
        parser.add_argument("-n", "--name", help="name", required=True)
        parser.add_argument("-b", "--body", help="request body", required=True)
        parser.set_defaults(
            func=lambda args: console.log(
                Caches(args, args.org, args.name)
                .create_a_cache_in_an_environment(args.environment, args.body)
                .text
            )
        )

    def _build_get_information_about_a_cache_argument(self):
        parser = self._parser_caches.add_parser(
            "get",
            aliases=["get-information-about-a-cache"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Gets information about a cache. The response might contain a property named persistent. That property is no longer used by Edge.",
        )
        parser.add_argument("-n", "--name", help="name", required=True)
        parser.set_defaults(
            func=lambda args: console.log(
                Caches(args, args.org, args.name)
                .get_information_about_a_cache(args.environment)
                .text
            )
        )

    def _build_list_caches_in_an_environment_argument(self):
        parser = self._parser_caches.add_parser(
            "list",
            aliases=["list-caches-in-an-environment"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
                self._prefix_parser(),
            ],
            help="List caches in an environment.",
        )
        parser.set_defaults(
            func=lambda args: console.log(
                Caches(args, args.org, None).list_caches_in_an_environment(
                    args.environment, prefix=args.prefix
                )
            )
        )

    def _build_update_a_cache_in_an_environment_argument(self):
        parser = self._parser_caches.add_parser(
            "update",
            aliases=["update-a-cache-in-an-environment"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Updates a cache in an environment. You must specify the complete definition of the cache, including the properties that you want to change and the ones that retain their current value. Any properties omitted from the request body are reset to their default value. Use Get information about a cache to obtain an object containing the current value of all properties, then change only those that you want to update.",
        )
        parser.add_argument("-n", "--name", help="name", required=True)
        parser.add_argument("-b", "--body", help="request body", required=True)
        parser.set_defaults(
            func=lambda args: console.log(
                Caches(args, args.org, args.name)
                .update_a_cache_in_an_environment(args.environment, args.body)
                .text
            )
        )

    def _build_delete_a_cache_argument(self):
        parser = self._parser_caches.add_parser(
            "delete",
            aliases=["delete-a-cache"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Deletes a cache.",
        )
        parser.add_argument("-n", "--name", help="name", required=True)
        parser.set_defaults(
            func=lambda args: console.log(
                Caches(args, args.org, args.name).delete_a_cache(args.environment).text
            )
        )

    def _build_push_cache_argument(self):
        parser = self._parser_caches.add_parser(
            "push",
            aliases=["push-cache"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
                self._file_parser(),
            ],
            help="Push Cache to Apigee. This will create/update a Cache.",
        )
        parser.set_defaults(
            func=lambda args: Caches(args, args.org, None).push_cache(
                args.environment, args.file
            )
        )

    def _create_parser(self):
        self._build_clear_all_cache_entries_argument()
        self._build_clear_a_cache_entry_argument()
        self._build_create_a_cache_in_an_environment_argument()
        self._build_get_information_about_a_cache_argument()
        self._build_list_caches_in_an_environment_argument()
        self._build_update_a_cache_in_an_environment_argument()
        self._build_delete_a_cache_argument()
        self._build_push_cache_argument()



import argparse
import builtins


class SilentParser:
    def __init__(self):
        self._parent_parser = argparse.ArgumentParser(add_help=False)
        self._build_silent_argument()
        self._silent = self._parent_parser.parse_known_args()[0].silent
        builtins.APIGEE_CLI_TOGGLE_SILENT = self._silent
        self._create_parser()

    @property
    def parent_parser(self):
        return self._parent_parser

    @parent_parser.setter
    def parent_parser(self, value):
        self._parent_parser = value

    def __call__(self):
        return self._parent_parser

    def _build_silent_argument(self):
        self._parent_parser.add_argument(
            # "-s",
            "--silent",
            action="store_true",
            help="toggle silent output",
            required=False,
        )

    def _create_parser(self):
        pass
